#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2Color, b2Globals, b2PolygonShape, b2Sweep, b2TimeOfImpact)


class TimeOfImpact (Framework):
    name = "Time of Impact"
    description = ("See the source code for more information."
                   "No additional controls.")

    def __init__(self):
        super(TimeOfImpact, self).__init__()

        # The two shapes to check for time of impact
        self.shapeA = b2PolygonShape(box=(0.2, 1, (0.5, 1), 0))
        self.shapeB = b2PolygonShape(box=(2, 0.1))

    def Step(self, settings):
        super(TimeOfImpact, self).Step(settings)

        # b2Sweep describes the motion of a body/shape for TOI computation.
        # Shapes are defined with respect to the body origin, which may no
        # coincide with the center of mass. However, to support dynamics we
        # must interpolate the center of mass position.
        sweepA = b2Sweep(c0=(0, 0), c=(0, 0),
                         a=0, a0=0,
                         localCenter=(0, 0))

        # The parameters of the sweep are defined as follows:
        # localCenter - local center of mass position
        # c0, c       - center world positions
        # a0, a       - world angles
        # t0          - time interval = [t0,1], where t0 is in [0,1]

        sweepB = b2Sweep(c0=(-0.20382018, 2.1368704),
                         a0=-3.1664171,
                         c=(-0.26699525, 2.3552670),
                         a=-3.3926492,
                         localCenter=(0, 0))

        type_, time_of_impact = b2TimeOfImpact(shapeA=self.shapeA,
                                               shapeB=self.shapeB,
                                               sweepA=sweepA, sweepB=sweepB,
                                               tMax=1.0)

        # Alternative pybox2d syntax (no kwargs):
        #  type_, t = b2TimeOfImpact(self.shapeA, self.shapeB, sweepA, sweepB, 1.0)
        #
        # And even uglier:
        #  input=b2TOIInput(proxyA=b2DistanceProxy(shape=self.shapeA), proxyB=b2DistanceProxy(shape=self.shapeB), sweepA=sweepA, sweepB=sweepB, tMax=1.0)
        #  type_, t = b2TimeOfImpact(input)

        self.Print("TOI = %g" % time_of_impact)
        self.Print("max toi iters = %d, max root iters = %d" %
                   (b2Globals.b2_toiMaxIters, b2Globals.b2_toiMaxRootIters))

        # Draw the shapes at their current position (t=0)
        # shapeA (the vertical polygon)
        transform = sweepA.GetTransform(0)
        self.renderer.DrawPolygon([self.renderer.to_screen(transform * v)
                                   for v in self.shapeA.vertices],
                                  b2Color(0.9, 0.9, 0.9))

        # shapeB (the horizontal polygon)
        transform = sweepB.GetTransform(0)
        self.renderer.DrawPolygon([self.renderer.to_screen(transform * v)
                                   for v in self.shapeB.vertices],
                                  b2Color(0.5, 0.9, 0.5))

        # localPoint=(2, -0.1)
        # rB = transform * localPoint - sweepB.c0
        # wB = sweepB.a - sweepB.a0
        # vB = sweepB.c - sweepB.c0
        # v = vB + b2Cross(wB, rB)

        # Now, draw shapeB in a different color when they would collide (i.e.,
        # at t=time of impact) This shows that the polygon would rotate upon
        # collision
        transform = sweepB.GetTransform(time_of_impact)
        self.renderer.DrawPolygon([self.renderer.to_screen(transform * v)
                                   for v in self.shapeB.vertices],
                                  b2Color(0.5, 0.7, 0.9))

        # And finally, draw shapeB at t=1.0, where it would be if it did not
        # collide with shapeA In this case, time_of_impact = 1.0, so these
        # become the same polygon.
        transform = sweepB.GetTransform(1.0)
        self.renderer.DrawPolygon([self.renderer.to_screen(transform * v)
                                   for v in self.shapeB.vertices],
                                  b2Color(0.9, 0.5, 0.5))

if __name__ == "__main__":
    main(TimeOfImpact)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import sys

class testBasic (unittest.TestCase):
#    def setUp(self):
#        pass

    def test_import(self):
        try:
            import Box2D
        except ImportError:
            self.fail("Unable to import Box2D library (%s)" % sys.exc_info()[1])

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from Box2D import *
from math import cos, sin
import sys

class ContactListener(b2ContactListener):
    pass

class testPolyshape (unittest.TestCase):
    def setUp(self):
        pass

    def dotest(self, world, v):
        body = world.CreateDynamicBody(position=(0,0),
                        shapes=b2PolygonShape(vertices=v) )
        for v1, v2 in zip(v, body.fixtures[0].shape.vertices):
            if v1 != v2:
                raise Exception('Vertices before and after creation unequal. Before and after zipped=%s'
                        % zip(v, body.fixtures[0].shape.vertices))

    def test_vertices(self):

        body = None
        self.cont_list=ContactListener()
        world = b2World(gravity=(0,-10), doSleep=True, contactListener=self.cont_list)

        try:
            # bad vertices list
            body = world.CreateDynamicBody(
                            position=(0,4),
                            shapes=[b2PolygonShape(vertices=(2,1)),
                                    b2PolygonShape(box=(2,1))
                                   ]
                             )
        except ValueError:
            pass # good
        else:
            raise Exception("Should have failed with ValueError / length 1")

        self.dotest(world, [(1,0),(1,1),(-1,1)] )
        self.dotest(world, [b2Vec2(1,0),(1,1),b2Vec2(-1,1)] )
        try:
            self.dotest(world, [(0,1,5),(1,1)] )
        except ValueError:
            pass # good
        else:
            raise Exception("Should have failed with ValueError / length 3")

        pi=b2_pi
        n=b2_maxPolygonVertices

        # int so floating point representation inconsistencies
        # don't make the vertex check fail
        v = [(int(20*cos(x*2*pi/n)), int(20*sin(x*2*pi/n))) for x in range(n)]
        self.dotest(world, v)

        try:
            self.dotest(world, [(0,1)]*(b2_maxPolygonVertices+1) )
        except ValueError:
            pass # good
        else:
            raise Exception("Should have failed with ValueError / max+1")

        # convex hull is used now, this is no longer valid
        #try:
        #    shape=b2PolygonShape(vertices=[(1,0),(0,-1),(-1,0)] )
        #except ValueError:
        #    pass # good, not convex
        #else:
        #    raise Exception("Should have failed with ValueError / checkpolygon")

        shape=b2PolygonShape(vertices=[(0,0), (0,1), (-1,0)] )
        temp=shape.valid

    def checkAlmostEqual(self, v1, v2, msg, places=3):
        if hasattr(v1, '__len__'):
            for i, (a, b) in enumerate(zip(v1, v2)):
                self.assertAlmostEqual(a, b, places=places,
                        msg="(index %d) %s, a=%f b=%f from %s, %s" % (i, msg, a, b, v1, v2))
        else:
            self.assertAlmostEqual(v1, v2, places=places,
                    msg="%s, a=%f b=%f" % (msg, v1, v2))

    def test_distance(self):
        # Transform A -- a simple translation/offset of (0,-0.2)
        self.transformA = b2Transform()
        self.transformA.SetIdentity()
        self.transformA.position = (0, -0.2)

        # Transform B -- a translation and a rotation
        self.transformB = b2Transform()
        self.positionB = b2Vec2(12.017401,0.13678508)
        self.angleB = -0.0109265
        self.transformB.Set(self.positionB, self.angleB)

        # The two shapes, transformed by the respective transform[A,B]
        self.polygonA = b2PolygonShape(box=(10,0.2))
        self.polygonB = b2PolygonShape(box=(2,0.1))

        # Calculate the distance between the two shapes with the specified transforms
        dist_result=b2Distance(shapeA=self.polygonA,
                               transformA=self.transformA,
                               idxA=0,
                               shapeB=self.polygonB,
                               transformB=self.transformB,
                               idxB=0,
                               useRadii=True)
        self.checkAlmostEqual(dist_result[0], (10, 0.01), 'point A', places=2)
        self.checkAlmostEqual(dist_result[1], (10, 0.05), 'point B', places=1)
        self.checkAlmostEqual(dist_result[2], 0.04, 'distance', places=2)
        assert(dist_result[3] == 2)

        input_ = b2DistanceInput(proxyA=b2DistanceProxy(self.polygonA, 0),
                                 transformA=self.transformA,
                                 proxyB=b2DistanceProxy(self.polygonB, 0),
                                 transformB=self.transformB,
                                 useRadii=True)

        assert(dist_result == b2Distance(input_))


if __name__ == '__main__':
    unittest.main()




"""
"""

import traceback

from .const import *
from .button import Button
from .basic import Label, Image
from .table import Table

class Select(Table):
    """A combo dropdown box widget.
    
    Example:
        w = Select(value="goats")
        w.add("Cats","cats")
        w.add("Goats","goats")
        w.add("Dogs","Dogs")
        w.value = 'dogs' #changes the value from goats to dogs
    
    """

    # The drop-down arrow button for the selection widget
    top_arrow = None
    # A button displaying the currently selected item
    top_selection = None
    # The first option added to the selector
    firstOption = None
    # The PGU table of options
    options = None
    _value = None

    def __init__(self,value=None,**params):
        params.setdefault('cls','select')
        Table.__init__(self,**params)
        
        label = Label(" ",cls=self.cls+".option.label")
        self.top_selected = Button(label, cls=self.cls+".selected")
        Table.add(self,self.top_selected) #,hexpand=1,vexpand=1)#,0,0)
        
        self.top_arrow = Button(Image(self.style.arrow), cls=self.cls+".arrow")
        Table.add(self,self.top_arrow) #,hexpand=1,vexpand=1) #,1,0)
        
        self.options = Table(cls=self.cls+".options")
        self.options.connect(BLUR,self._close,None)
        self.options.name = "pulldown-table"
        
        self.values = []
        self.value = value

    def resize(self,width=None,height=None):
        max_w,max_h = 0,0
        for w in self.options.widgets:
            w.rect.w,w.rect.h = w.resize()
            max_w,max_h = max(max_w,w.rect.w),max(max_h,w.rect.h)
        
        #xt,xr,xb,xl = self.top_selected.getspacing()
        self.top_selected.style.width = max_w #+ xl + xr
        self.top_selected.style.height = max_h #+ xt + xb
        
        self.top_arrow.connect(CLICK,self._open,None)
        self.top_selected.connect(CLICK,self._open,None)
        
        w,h = Table.resize(self,width,height)
        
        self.options.style.width = w
        #HACK: sort of, but not a big one..
        self.options.resize()
        
        return w,h
        
    def _open(self,value):
        opts = self.options
        
        opts.rect.w, opts.rect.h = opts.resize()
        
#        y = self.rect.y
#        c = self.container
#        while hasattr(c, 'container'):
#            y += c.rect.y
#            if (not c.container): 
#                break
#            c = c.container
            
#        if y + self.rect.h + opts.rect.h <= c.rect.h: #down
#            dy = self.rect.y + self.rect.h
#        else: #up
#            dy = self.rect.y - self.rect.h

        opts.rect.w, opts.rect.h = opts.resize()

        # TODO - make sure there is enough space to open down
        # ...
        yp = self.rect.bottom-1

        self.container.open(opts, self.rect.x, yp)
        self.firstOption.focus()

        # TODO - this is a hack
        for opt in self.options.widgets:
            opt.repaint()

    def _close(self,value):
        self.options.close()
        self.top_selected.focus()
    
    def _setvalue(self,value):
        self.value = value._value
        if self.container:
            #self.chsize()
            #HACK: improper use of resize()
            #self.resize() #to recenter the new value, etc.
            pass
        #    #self._resize()
        
        self._close(None)
        #self.repaint() #this will happen anyways
        
    
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, val):
        mywidget = None
        for w in self.values:
            if w._value == val:
                mywidget = w
        oldval = self._value
        self._value = val
        if (oldval != val):
            self.send(CHANGE)
            self.repaint()
        if not mywidget:
            mywidget = Label(" ",cls=self.cls+".option.label")
        self.top_selected.value = mywidget
        
    
    def add(self,w,value=None):
        """Add a widget and associated value to the dropdown box."""
        
        if type(w) == str: w = Label(w,cls=self.cls+".option.label")
        
        w.style.align = -1
        btn = Button(w,cls=self.cls+".option")
        btn.connect(CLICK,self._setvalue,w)
        
        self.options.tr()
        self.options.add(btn)
        
        if (not self.firstOption):
            self.firstOption = btn
        
        if value != None: w._value = value
        else: w._value = w
        if self.value == w._value:
            self.top_selected.value = w
        self.values.append(w)




#!/usr/bin/python
#
# C++ version copyright 2010 Erin Catto http://www.gphysics.com
# Python version copyright 2010 Ken Lauer / sirkne at gmail dot com
# 
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.
# 
from .Box2D import *
__version__ = '2.3.10'
__version_info__ = (2,3,10)
__license__ = 'zlib'


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, Keys, main)
from random import random
from math import sqrt, sin, cos

from Box2D import (b2BodyDef, b2CircleShape, b2Color, b2EdgeShape,
                   b2FixtureDef, b2PolygonShape, b2RayCastCallback, b2Vec2,
                   b2_dynamicBody, b2_pi)


class RayCastClosestCallback(b2RayCastCallback):
    """This callback finds the closest hit"""

    def __repr__(self):
        return 'Closest hit'

    def __init__(self, **kwargs):
        b2RayCastCallback.__init__(self, **kwargs)
        self.fixture = None
        self.hit = False

    def ReportFixture(self, fixture, point, normal, fraction):
        '''
        Called for each fixture found in the query. You control how the ray
        proceeds by returning a float that indicates the fractional length of
        the ray. By returning 0, you set the ray length to zero. By returning
        the current fraction, you proceed to find the closest point. By
        returning 1, you continue with the original ray clipping. By returning
        -1, you will filter out the current fixture (the ray will not hit it).
        '''
        self.hit = True
        self.fixture = fixture
        self.point = b2Vec2(point)
        self.normal = b2Vec2(normal)
        # NOTE: You will get this error:
        #   "TypeError: Swig director type mismatch in output value of
        #    type 'float32'"
        # without returning a value
        return fraction


class RayCastAnyCallback(b2RayCastCallback):
    """This callback finds any hit"""

    def __repr__(self):
        return 'Any hit'

    def __init__(self, **kwargs):
        b2RayCastCallback.__init__(self, **kwargs)
        self.fixture = None
        self.hit = False

    def ReportFixture(self, fixture, point, normal, fraction):
        self.hit = True
        self.fixture = fixture
        self.point = b2Vec2(point)
        self.normal = b2Vec2(normal)
        return 0.0


class RayCastMultipleCallback(b2RayCastCallback):
    """This raycast collects multiple hits."""

    def __repr__(self):
        return 'Multiple hits'

    def __init__(self, **kwargs):
        b2RayCastCallback.__init__(self, **kwargs)
        self.fixture = None
        self.hit = False
        self.points = []
        self.normals = []

    def ReportFixture(self, fixture, point, normal, fraction):
        self.hit = True
        self.fixture = fixture
        self.points.append(b2Vec2(point))
        self.normals.append(b2Vec2(normal))
        return 1.0


class Raycast (Framework):
    name = "Raycast"
    description = "Press 1-5 to drop stuff, d to delete, m to switch callback modes"
    p1_color = b2Color(0.4, 0.9, 0.4)
    s1_color = b2Color(0.8, 0.8, 0.8)
    s2_color = b2Color(0.9, 0.9, 0.4)

    def __init__(self):
        super(Raycast, self).__init__()

        self.world.gravity = (0, 0)
        # The ground
        ground = self.world.CreateBody(
            shapes=b2EdgeShape(vertices=[(-40, 0), (40, 0)])
        )

        # The various shapes
        w = 1.0
        b = w / (2.0 + sqrt(2.0))
        s = sqrt(2.0) * b

        self.shapes = [
            b2PolygonShape(vertices=[(-0.5, 0), (0.5, 0), (0, 1.5)]),
            b2PolygonShape(vertices=[(-0.1, 0), (0.1, 0), (0, 1.5)]),
            b2PolygonShape(
                vertices=[(0.5 * s, 0), (0.5 * w, b), (0.5 * w, b + s),
                          (0.5 * s, w), (-0.5 * s, w), (-0.5 * w, b + s),
                          (-0.5 * w, b), (-0.5 * s, 0.0)]
            ),
            b2PolygonShape(box=(0.5, 0.5)),
            b2CircleShape(radius=0.5),
        ]
        self.angle = 0

        self.callbacks = [RayCastClosestCallback,
                          RayCastAnyCallback, RayCastMultipleCallback]
        self.callback_class = self.callbacks[0]

    def CreateShape(self, shapeindex):
        try:
            shape = self.shapes[shapeindex]
        except IndexError:
            return

        pos = (10.0 * (2.0 * random() - 1.0), 10.0 * (2.0 * random()))
        defn = b2BodyDef(
            type=b2_dynamicBody,
            fixtures=b2FixtureDef(shape=shape, friction=0.3),
            position=pos,
            angle=(b2_pi * (2.0 * random() - 1.0)),
        )

        if isinstance(shape, b2CircleShape):
            defn.angularDamping = 0.02

        self.world.CreateBody(defn)

    def DestroyBody(self):
        for body in self.world.bodies:
            if not self.world.locked:
                self.world.DestroyBody(body)
            break

    def Keyboard(self, key):
        if key in (Keys.K_1, Keys.K_2, Keys.K_3, Keys.K_4, Keys.K_5):
            self.CreateShape(key - Keys.K_1)
        elif key == Keys.K_d:
            self.DestroyBody()
        elif key == Keys.K_m:
            idx = ((self.callbacks.index(self.callback_class) + 1) %
                   len(self.callbacks))
            self.callback_class = self.callbacks[idx]

    def Step(self, settings):
        super(Raycast, self).Step(settings)

        def draw_hit(cb_point, cb_normal):
            cb_point = self.renderer.to_screen(cb_point)
            head = b2Vec2(cb_point) + 0.5 * cb_normal

            cb_normal = self.renderer.to_screen(cb_normal)
            self.renderer.DrawPoint(cb_point, 5.0, self.p1_color)
            self.renderer.DrawSegment(point1, cb_point, self.s1_color)
            self.renderer.DrawSegment(cb_point, head, self.s2_color)

        # Set up the raycast line
        length = 11
        point1 = b2Vec2(0, 10)
        d = (length * cos(self.angle), length * sin(self.angle))
        point2 = point1 + d

        callback = self.callback_class()

        self.world.RayCast(callback, point1, point2)

        # The callback has been called by this point, and if a fixture was hit it will have been
        # set to callback.fixture.
        point1 = self.renderer.to_screen(point1)
        point2 = self.renderer.to_screen(point2)

        if callback.hit:
            if hasattr(callback, 'points'):
                for point, normal in zip(callback.points, callback.normals):
                    draw_hit(point, normal)
            else:
                draw_hit(callback.point, callback.normal)
        else:
            self.renderer.DrawSegment(point1, point2, self.s1_color)

        self.Print("Callback: %s" % callback)
        if callback.hit:
            self.Print("Hit")

        if not settings.pause or settings.singleStep:
            self.angle += 0.25 * b2_pi / 180

if __name__ == "__main__":
    main(Raycast)



import pygame
from pygame.locals import *

from .const import *
from . import widget
from . import table
from . import basic
from . import pguglobals

_SLIDER_HORIZONTAL = 0
_SLIDER_VERTICAL = 1

class _slider(widget.Widget):
    _value = None

    def __init__(self,value,orient,min,max,size,step=1,**params):
        params.setdefault('cls','slider')
        widget.Widget.__init__(self,**params)
        self.min,self.max,self.value,self.orient,self.size,self.step = min,max,value,orient,size,step
        
    
    def paint(self,s):
        
        self.value = self.value
        r = pygame.rect.Rect(0,0,self.style.width,self.style.height)
        if self.orient == _SLIDER_HORIZONTAL:
            r.x = (self.value-self.min) * (r.w-self.size) / max(1,self.max-self.min);
            r.w = self.size;
        else:
            r.y = (self.value-self.min) * (r.h-self.size) / max(1,self.max-self.min);
            r.h = self.size;
            
        self.bar = r
        
        pguglobals.app.theme.render(s,self.style.bar,r)
    
    def event(self,e):
        used = None
        r = pygame.rect.Rect(0,0,self.style.width,self.style.height)
        adj = 0
        if e.type == ENTER: self.repaint()
        elif e.type == EXIT: self.repaint()
        elif e.type == MOUSEBUTTONDOWN:
            if self.bar.collidepoint(e.pos):
                self.grab = e.pos[0],e.pos[1]
                self.grab_value = self.value
            else:
                x,y,adj = e.pos[0],e.pos[1],1
                self.grab = None
            self.repaint()
        elif e.type == MOUSEBUTTONUP:
            #x,y,adj = e.pos[0],e.pos[1],1
            self.repaint()
        elif e.type == MOUSEMOTION:
            if 1 in e.buttons and self.container.myfocus is self:
                if self.grab != None:
                    rel = e.pos[0]-self.grab[0],e.pos[1]-self.grab[1]
                    if self.orient == _SLIDER_HORIZONTAL:
                        d = (r.w - self.size)
                        if d != 0: self.value = self.grab_value + ((self.max-self.min) * rel[0] / d)
                    else:
                        d = (r.h - self.size)
                        if d != 0: self.value = self.grab_value + ((self.max-self.min) * rel[1] / d)
                else:
                    x,y,adj = e.pos[0],e.pos[1],1
                    
        elif e.type is KEYDOWN:
            if self.orient == _SLIDER_HORIZONTAL and e.key == K_LEFT:
                self.value -= self.step
                used = True
            elif self.orient == _SLIDER_HORIZONTAL and e.key == K_RIGHT:
                self.value += self.step
                used = True
            elif self.orient == _SLIDER_VERTICAL and e.key == K_UP:
                self.value -= self.step
                used = True
            elif self.orient == _SLIDER_VERTICAL and e.key == K_DOWN:
                self.value += self.step
                used = True

        if adj:
            if self.orient == _SLIDER_HORIZONTAL:
                d = self.size/2 - (r.w/(self.max-self.min+1))/2
                self.value = (x-d) * (self.max-self.min) / (r.w-self.size+1) + self.min
            else:
                d = self.size/2 - (r.h/(self.max-self.min+1))/2
                self.value = (y-d) * (self.max-self.min) / (r.h-self.size+1) + self.min
                
        self.pcls = ""
        if self.container.myhover is self: self.pcls = "hover"
        if (self.container.myfocus is self and 1 in pygame.mouse.get_pressed()): self.pcls = "down"
        
        return used

    # TODO - replace this with property functions and setters
    def __setattr__(self,k,v):
        if k == 'value':
            v = int(v)
            v = max(v,self.min)
            v = min(v,self.max)
        _v = self.__dict__.get(k,NOATTR)
        self.__dict__[k]=v
        if k == 'value' and _v != NOATTR and _v != v: 
            self.send(CHANGE)
            self.repaint()
        if hasattr(self,'size'):
            sz = min(self.size,max(self.style.width,self.style.height))
            sz = max(sz,min(self.style.width,self.style.height))
            self.__dict__['size'] = sz
            #self.size = sz
        if hasattr(self,'max') and hasattr(self,'min'):
            if self.max < self.min: self.max = self.min

#    @property
#    def value(self):
#        return self._value
#
#    @value.setter
#    def value(self, val):
#        val = int(val)
#        val = max(val, self.min)
#        val = min(val, self.max)
#
#        oldval = self._value
#        self._value = val
#        if (oldval != val):
#            self.send(CHANGE)
#            self.repaint()
#            
#        if hasattr(self,'size'):
#            sz = min(self.size,max(self.style.width,self.style.height))
#            sz = max(sz,min(self.style.width,self.style.height))
#            self.size = sz
#            
#        if hasattr(self,'max') and hasattr(self,'min'):
#            if self.max < self.min: self.max = self.min
    

class VSlider(_slider):
    """A verticle slider."""

    def __init__(self,value,min,max,size,step=1,**params):
        """Construct a veritcal slider widget.

        Arguments:
            value -- the default position of the slider, between min and max
            min -- the minimum value for the slider
            max -- the maximum value
            size -- the length of the slider bar in pixels
            step -- how much to jump when using the keyboard

        """
        params.setdefault('cls','vslider')
        _slider.__init__(self,value,_SLIDER_VERTICAL,min,max,size,step,**params)

class HSlider(_slider):
    """A horizontal slider."""

    def __init__(self,value,min,max,size,step=1,**params):
        params.setdefault('cls','hslider')
        _slider.__init__(self,value,_SLIDER_HORIZONTAL,min,max,size,step,**params)

class HScrollBar(table.Table):
    """A horizontal scroll bar."""

    def __init__(self,value,min,max,size,step=1,**params):
        params.setdefault('cls','hscrollbar')
        
        table.Table.__init__(self,**params)
        
        self.slider = _slider(value,_SLIDER_HORIZONTAL,min,max,size,step=step,cls=self.cls+'.slider')
        
        self.minus = basic.Image(self.style.minus)
        self.minus.connect(MOUSEBUTTONDOWN,self._click,-1)
        self.slider.connect(CHANGE,self.send,CHANGE)
        
        self.minus2 = basic.Image(self.style.minus)
        self.minus2.connect(MOUSEBUTTONDOWN,self._click,-1)
        
        self.plus = basic.Image(self.style.plus)
        self.plus.connect(MOUSEBUTTONDOWN,self._click,1)
        
        self.size = size
        
    def _click(self,value):
        self.slider.value += self.slider.step*value
        
    def resize(self,width=None,height=None):
        self.clear()
        self.tr()
        
        w = self.style.width
        h = self.slider.style.height
        ww = 0
        
        if w > (h*2 + self.minus.style.width+self.plus.style.width):
            self.td(self.minus)
            ww += self.minus.style.width
        
        self.td(self.slider)
        
        if w > (h*2 + self.minus.style.width+self.minus2.style.width+self.plus.style.width):
            self.td(self.minus2)
            ww += self.minus2.style.width
        
        if w > (h*2 + self.minus.style.width+self.plus.style.width):
            self.td(self.plus)
            ww += self.plus.style.width
            
            
        #HACK: handle theme sizing properly
        xt,xr,xb,xl = pguglobals.app.theme.getspacing(self.slider)
        ww += xr+xl

        self.slider.style.width = self.style.width - ww
        setattr(self.slider,'size',self.size * self.slider.style.width / max(1,self.style.width))
        #self.slider.size = self.size * self.slider.style.width / max(1,self.style.width)
        return table.Table.resize(self,width,height)

    @property
    def min(self):
        return self.slider.min

    @min.setter
    def min(self, value):
        self.slider.min = value

    @property
    def max(self):
        return self.slider.max

    @max.setter
    def max(self, value):
        self.slider.max = value

    @property
    def value(self):
        return self.slider.value

    @value.setter
    def value(self, value):
        self.slider.value = value

    @property
    def step(self):
        return self.slider.step

    @step.setter
    def step(self, value):
        self.slider.step = value
        
#    def __setattr__(self,k,v):
#        if k in ('min','max','value','step'):
#            return setattr(self.slider,k,v)
#        self.__dict__[k]=v
            
#    def __getattr__(self,k):
#        if k in ('min','max','value','step'):
#            return getattr(self.slider,k)
#        return table.Table.__getattr__(self,k) #self.__dict__[k]

class VScrollBar(table.Table):
    """A vertical scroll bar."""

    def __init__(self,value,min,max,size,step=1,**params):
        params.setdefault('cls','vscrollbar')
        
        table.Table.__init__(self,**params)
        
        self.minus = basic.Image(self.style.minus)
        self.minus.connect(MOUSEBUTTONDOWN,self._click,-1)
        
        self.minus2 = basic.Image(self.style.minus)
        self.minus2.connect(MOUSEBUTTONDOWN,self._click,-1)
        
        self.plus = basic.Image(self.style.plus)
        self.plus.connect(MOUSEBUTTONDOWN,self._click,1)
        
        self.slider = _slider(value,_SLIDER_VERTICAL,min,max,size,step=step,cls=self.cls+'.slider')
        self.slider.connect(CHANGE,self.send,CHANGE)
        
        self.size = size
        
    def _click(self,value):
        self.slider.value += self.slider.step*value
        
    def resize(self,width=None,height=None):
        self.clear()
        
        h = self.style.height
        w = self.slider.style.width
        hh = 0
        
        if h > (w*2 + self.minus.style.height+self.plus.style.height):
            self.tr()
            self.td(self.minus)
            hh += self.minus.style.height
        
        self.tr()
        self.td(self.slider)
        
        if h > (w*2 + self.minus.style.height+self.minus2.style.height+self.plus.style.height):
            self.tr()
            self.td(self.minus2)
            hh += self.minus2.style.height
        
        if h > (w*2 + self.minus.style.height+self.plus.style.height):
            self.tr()
            self.td(self.plus)
            hh += self.plus.style.height
            
            
        #HACK: handle theme sizing properly
        xt,xr,xb,xl = pguglobals.app.theme.getspacing(self.slider)
        hh += xt+xb

        self.slider.style.height = self.style.height - hh
        setattr(self.slider,'size',self.size * self.slider.style.height / max(1,self.style.height))
        return table.Table.resize(self,width,height)
        
    def __setattr__(self,k,v):
        if k in ('min','max','value','step'):
            return setattr(self.slider,k,v)
        self.__dict__[k]=v
            
    def __getattr__(self,k):
        if k in ('min','max','value','step'):
            return getattr(self.slider,k)
        return table.Table.__getattr__(self,k)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2EdgeShape, b2FixtureDef, b2PolygonShape, b2Random)


class Bullet (Framework):
    name = "Bullet"
    description = 'A test for very fast moving objects (bullets)'

    def __init__(self):
        super(Bullet, self).__init__()

        ground = self.world.CreateStaticBody(
            position=(0, 0),
            shapes=[b2EdgeShape(vertices=[(-10, 0), (10, 0)]),
                    b2PolygonShape(box=(0.2, 1, (0.5, 1), 0))]
        )

        self._x = 0.20352793
        self.body = self.world.CreateDynamicBody(
            position=(0, 4),
            fixtures=b2FixtureDef(
                shape=b2PolygonShape(box=(2, 0.1)), density=1.0),
        )

        self.bullet = self.world.CreateDynamicBody(
            position=(self._x, 10),
            bullet=True,
            fixtures=b2FixtureDef(shape=b2PolygonShape(
                box=(0.25, 0.25)), density=100.0),
            linearVelocity=(0, -50)
        )

    def Launch(self):
        self.body.transform = [(0, 4), 0]
        self.body.linearVelocity = (0, 0)
        self.body.angularVelocity = 0

        self.x = b2Random()
        self.bullet.transform = [(self.x, 10), 0]
        self.bullet.linearVelocity = (0, -50)
        self.bullet.angularVelocity = 0

    def Step(self, settings):
        super(Bullet, self).Step(settings)

        if (self.stepCount % 60) == 0:
            self.Launch()

if __name__ == "__main__":
    main(Bullet)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from Box2D import b2Vec2, b2Vec3, b2Mat22, b2Mat33

class testMatrix (unittest.TestCase):
    def checkAlmostEqual(self, v1, v2, msg):
        for a, b in zip(v1, v2):
            self.assertAlmostEqual(a, b, places=3,
                    msg="%s, a=%f b=%f from %s, %s" % (msg, a, b, v1, v2))

    def test_mat22_identity(self):
        i2 = b2Mat22()
        self.checkAlmostEqual(i2.col1, (1.0, 0.0), msg='mat22 col1')
        self.checkAlmostEqual(i2.col2, (0.0, 1.0), msg='mat22 col2')

    def test_matrix(self):
        x, y, z = 1.0, 2.0, 3.0
        v2 = b2Vec2(x, y)

        self.checkAlmostEqual(v2.skew, (-v2.y, v2.x), msg='skew')

        m2 = b2Mat22((x, y), (y, x))
        # Note that you can't do:
        # m2 = b2Mat22(col1=(x, y), col2=(y, x))
        # as SWIG will not allow the kwargs option to be used when there are multiple constructors

        m = m2 + m2
        self.checkAlmostEqual(m.col1, (x+x, y+y), msg='b2Mat22 +')
        self.checkAlmostEqual(m.col2, (y+y, x+x), msg='b2Mat22 +')
        m = m2 - m2
        self.checkAlmostEqual(m.col1, (0,0), msg='b2Mat22 -')
        self.checkAlmostEqual(m.col2, (0,0), msg='b2Mat22 -')
        # x y  * x
        # y x    y
        v = m2 * v2
        self.checkAlmostEqual(v, (x*x + y*y, y*x + y*x), msg='b2Mat22 * b2Vec2')
        i=m2.inverse
        i=m2.angle
        m = m2 * m2
        self.checkAlmostEqual(m.col1, (x*x + y*y, y*x + y*x), msg='b2Mat22 * b2Mat22')
        self.checkAlmostEqual(m.col2, (x*y + y*x, y*y + x*x), msg='b2Mat22 * b2Mat22')

        m2 += m2
        self.checkAlmostEqual(m2.col1, (x+x, y+y), msg='b2Mat22 +=')
        self.checkAlmostEqual(m2.col2, (y+y, x+x), msg='b2Mat22 +=')
        m2 -= m2
        self.checkAlmostEqual(m2.col1, (0,0), msg='b2Mat22 -=')
        self.checkAlmostEqual(m2.col2, (0,0), msg='b2Mat22 -=')

    def test_mat33_identity(self):
        i3 = b2Mat33()
        self.checkAlmostEqual(i3.col1, (1.0, 0.0, 0.0), msg='mat33 col1')
        self.checkAlmostEqual(i3.col2, (0.0, 1.0, 0.0), msg='mat33 col2')
        self.checkAlmostEqual(i3.col3, (0.0, 0.0, 1.0), msg='mat33 col3')

    def test_mat33(self):
        x, y, z = 1.0, 2.0, 3.0
        v3 = b2Vec3(x, y, z)
        m3 = b2Mat33((x, y, z), (z, y, x), (y, x, z))
        m = m3 + m3
        self.checkAlmostEqual(m.col1, (x+x, y+y, z+z), msg='b2Mat33 +')
        self.checkAlmostEqual(m.col2, (z+z, y+y, x+x), msg='b2Mat33 +')
        self.checkAlmostEqual(m.col3, (y+y, x+x, z+z), msg='b2Mat33 +')
        m = m3 - m3
        self.checkAlmostEqual(m.col1, (0,0,0), msg='b2Mat33 -')
        self.checkAlmostEqual(m.col2, (0,0,0), msg='b2Mat33 -')
        self.checkAlmostEqual(m.col3, (0,0,0), msg='b2Mat33 -')
        m3 += m3
        self.checkAlmostEqual(m3.col1, (x+x, y+y, z+z), msg='b2Mat33 +=')
        self.checkAlmostEqual(m3.col2, (z+z, y+y, x+x), msg='b2Mat33 +=')
        self.checkAlmostEqual(m3.col3, (y+y, x+x, z+z), msg='b2Mat33 +=')
        m3 -= m3
        self.checkAlmostEqual(m3.col1, (0,0,0), msg='b2Mat33 -=')
        self.checkAlmostEqual(m3.col2, (0,0,0), msg='b2Mat33 -=')
        self.checkAlmostEqual(m3.col3, (0,0,0), msg='b2Mat33 -=')


if __name__ == '__main__':
    unittest.main()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

# Contributed by Giorgos Giagas (giorgosg)
# - updated for 2.1 by Ken

from math import sin, cos, pi

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2CircleShape, b2FixtureDef)


def create_blob(world, center, radius, circle_radius=0.5, shape_num=24,
                angularDamping=0.5, linearDamping=0.5, friction=0.5,
                density=5.0, **kwargs):

    def get_pos(angle):
        return (cos(angle * pi / 180.0) * radius + center[0],
                sin(angle * pi / 180.0) * radius + center[1])

    circle = b2CircleShape(radius=circle_radius)
    fixture = b2FixtureDef(shape=circle, friction=friction, density=density,
                           restitution=0.0)

    bodies = [world.CreateDynamicBody(position=get_pos(
        i), fixtures=fixture) for i in range(0, 360, int(360 / shape_num))]
    joints = []

    prev_body = bodies[-1]
    for body in bodies:
        joint = world.CreateDistanceJoint(
            bodyA=prev_body,
            bodyB=body,
            anchorA=prev_body.position,
            anchorB=body.position,
            dampingRatio=10.0)

        joints.append(joint)
        prev_body = body

    return bodies, joints


def add_spring_force(bodyA, localA, bodyB, localB, force_k, friction,
                     desiredDist):
    worldA = bodyA.GetWorldPoint(localA)
    worldB = bodyB.GetWorldPoint(localB)
    diff = worldB - worldA

    # Find velocities of attach points
    velA = bodyA.linearVelocity - \
        bodyA.GetWorldVector(localA).cross(bodyA.angularVelocity)
    velB = bodyB.linearVelocity - \
        bodyB.GetWorldVector(localB).cross(bodyB.angularVelocity)

    vdiff = velB - velA
    dx = diff.Normalize()  # Normalizes diff and puts length into dx
    vrel = vdiff.x * diff.x + vdiff.y * diff.y
    forceMag = -force_k * (dx - desiredDist) - friction * vrel

    bodyB.ApplyForce(diff * forceMag, bodyA.GetWorldPoint(localA), True)
    bodyA.ApplyForce(diff * -forceMag, bodyB.GetWorldPoint(localB), True)


def blob_step(world, blob_bodies, radius, upward_force, move=0,
              spring_friction=5.0):
    body_count = len(blob_bodies)
    bodies1, bodies2 = (blob_bodies[:body_count // 2],
                        blob_bodies[body_count // 2:])
    for body1, body2 in zip(bodies1, bodies2):
        add_spring_force(body1, (0, 0), body2, (0, 0),
                         upward_force, spring_friction, radius * 2)
    if move:
        top_body = [(body.position.y, body) for body in blob_bodies]
        top_body.sort(key=lambda val: val[0])
        top_body = top_body[-1][1]
        top_body.ApplyForce((move, 0), top_body.position, True)


class GishTribute (Framework):
    name = "Tribute to Gish"
    description = 'Keys: Left (a), Right (d), Jump (w), Stop (s)'
    move = 0
    jump = 100

    def __init__(self):
        super(GishTribute, self).__init__()

        # The ground
        ground = self.world.CreateStaticBody()
        ground.CreateEdgeFixture(vertices=[(-50, 0), (50, 0)], friction=0.2)
        ground.CreateEdgeFixture(vertices=[(-50, 0), (-50, 10)], friction=0.2)
        ground.CreateEdgeFixture(vertices=[(50, 0), (50, 10)], friction=0.2)

        for i in range(2, 18, 2):
            body = self.world.CreateDynamicBody(position=(-10.1, i))
            body.CreatePolygonFixture(box=(3.0, 1.0), density=3.0)

        self.blob_radius = 2
        self.bodies, self.joints = create_blob(
            self.world, (-10, 50), self.blob_radius, circle_radius=0.5)

    def Keyboard(self, key):
        if key == Keys.K_w:
            self.jump = 10000
        elif key == Keys.K_a:
            self.move = -500
        elif key == Keys.K_d:
            self.move = 500
        elif key == Keys.K_s:
            self.move = 0
            self.jump = 100

    def Step(self, settings):
        Framework.Step(self, settings)

        blob_step(self.world, self.bodies,
                  self.blob_radius, self.jump, self.move)
        self.jump = 100

if __name__ == "__main__":
    main(GishTribute)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

"""
Global Keys:
    Space  - shoot projectile
    Z/X    - zoom
    Escape - quit

Other keys can be set by the individual test.

Mouse:
    Left click  - select/drag body (creates mouse joint)
    Right click - get body/fixture information
    Shift+Left  - drag to create a directed projectile
    Scroll      - zoom

"""
import string
import sys
import re

from PyQt4 import (QtGui, QtCore)
from PyQt4.QtGui import (QTableWidgetItem, QColor)
from PyQt4.QtCore import Qt

from Box2D import (b2AABB, b2CircleShape, b2Color, b2DistanceJoint,
                   b2EdgeShape, b2LoopShape, b2MouseJoint, b2Mul,
                   b2PolygonShape, b2PulleyJoint, b2Vec2)
from Box2D import (b2_pi, b2_staticBody, b2_kinematicBody)

from ..framework import (fwQueryCallback, FrameworkBase, Keys)
from .. import settings
from .pyqt4_gui import Ui_MainWindow


class Pyqt4Draw(object):
    """
    This debug drawing class differs from the other frameworks.  It provides an
    example of how to iterate through all the objects in the world and
    associate (in PyQt4's case) QGraphicsItems with them.

    While DrawPolygon and DrawSolidPolygon are not used for the core shapes in
    the world (DrawPolygonShape is), they are left in for compatibility with
    other frameworks and the tests.

    world_coordinate parameters are also left in for compatibility.  Screen
    coordinates cannot be used, as PyQt4 does the scaling and rotating for us.

    If you utilize this framework and need to add more items to the
    QGraphicsScene for a single step, be sure to add them to the temp_items
    array to be deleted on the next draw.
    """
    MAX_TIMES = 20
    axisScale = 0.4

    def __init__(self, test):
        self.test = test
        self.window = self.test.window
        self.scene = self.window.scene
        self.view = self.window.graphicsView
        self.item_cache = {}
        self.temp_items = []
        self.status_font = QtGui.QFont("Times", 10, QtGui.QFont.Bold)
        self.font_spacing = QtGui.QFontMetrics(self.status_font).lineSpacing()
        self.draw_idx = 0

    def StartDraw(self):
        for item in self.temp_items:
            self.scene.removeItem(item)
        self.temp_items = []

    def EndDraw(self):
        pass

    def SetFlags(self, **kwargs):
        """
        For compatibility with other debug drawing classes.
        """
        pass

    def DrawStringAt(self, x, y, str, color=None):
        item = QtGui.QGraphicsSimpleTextItem(str)
        if color is None:
            color = (255, 255, 255, 255)

        brush = QtGui.QBrush(QColor(255, 255, 255, 255))
        item.setFont(self.status_font)
        item.setBrush(brush)
        item.setPos(self.view.mapToScene(x, y))
        item.scale(1. / self.test._viewZoom, -1. / self.test._viewZoom)
        self.temp_items.append(item)

        self.scene.addItem(item)

    def DrawPoint(self, p, size, color):
        """
        Draw a single point at point p given a pixel size and color.
        """
        self.DrawCircle(p, size / self.test.viewZoom, color, drawwidth=0)

    def DrawAABB(self, aabb, color):
        """
        Draw a wireframe around the AABB with the given color.
        """
        line1 = self.scene.addLine(aabb.lowerBound.x, aabb.lowerBound.y,
                                   aabb.upperBound.x, aabb.lowerBound.y,
                                   pen=QtGui.QPen(QColor(*color.bytes)))
        line2 = self.scene.addLine(aabb.upperBound.x, aabb.upperBound.y,
                                   aabb.lowerBound.x, aabb.upperBound.y,
                                   pen=QtGui.QPen(QColor(*color.bytes)))
        self.temp_items.append(line1)
        self.temp_items.append(line2)

    def DrawSegment(self, p1, p2, color):
        """
        Draw the line segment from p1-p2 with the specified color.
        """
        line = self.scene.addLine(p1[0], p1[1], p2[0], p2[1],
                                  pen=QtGui.QPen(QColor(*color.bytes)))
        self.temp_items.append(line)

    def DrawTransform(self, xf):
        """
        Draw the transform xf on the screen
        """
        p1 = xf.position
        p2 = p1 + self.axisScale * xf.R.x_axis
        p3 = p1 + self.axisScale * xf.R.y_axis

        line1 = self.scene.addLine(p1[0], p1[1], p2[0], p2[1],
                                   pen=QtGui.QPen(QColor(255, 0, 0)))
        line2 = self.scene.addLine(p1[0], p1[1], p3[0], p3[1],
                                   pen=QtGui.QPen(QColor(0, 255, 0)))
        self.temp_items.append(line1)
        self.temp_items.append(line2)

    def DrawCircle(self, center, radius, color, drawwidth=1, shape=None):
        """
        Draw a wireframe circle given the center, radius, axis of orientation
        and color.
        """
        border_color = [c * 255 for c in color] + [255]
        pen = QtGui.QPen(QtGui.QColor(*border_color))
        ellipse = self.scene.addEllipse(center[0] - radius, center[1] - radius,
                                        radius * 2, radius * 2, pen=pen)
        self.temp_items.append(ellipse)

    def DrawSolidCircle(self, center, radius, axis, color, shape=None):
        """
        Draw a solid circle given the center, radius, axis of orientation and
        color.
        """
        border_color = color.bytes + [255]
        inside_color = (color / 2).bytes + [127]
        brush = QtGui.QBrush(QtGui.QColor(*inside_color))
        pen = QtGui.QPen(QtGui.QColor(*border_color))
        ellipse = self.scene.addEllipse(center[0] - radius, center[1] - radius,
                                        radius * 2, radius * 2, brush=brush,
                                        pen=pen)
        line = self.scene.addLine(center[0], center[1],
                                  (center[0] - radius * axis[0]),
                                  (center[1] - radius * axis[1]),
                                  pen=QtGui.QPen(QColor(255, 0, 0)))

        self.temp_items.append(ellipse)
        self.temp_items.append(line)

    def DrawPolygon(self, vertices, color, shape=None):
        """
        Draw a wireframe polygon given the world vertices vertices (tuples)
        with the specified color.
        """
        poly = QtGui.QPolygonF()
        pen = QtGui.QPen(QtGui.QColor(*color.bytes))

        for v in vertices:
            poly += QtCore.QPointF(*v)

        item = self.scene.addPolygon(poly, pen=pen)
        self.temp_items.append(item)

    def DrawSolidPolygon(self, vertices, color, shape=None):
        """
        Draw a filled polygon given the world vertices vertices (tuples) with
        the specified color.
        """
        poly = QtGui.QPolygonF()
        border_color = color.bytes + [255]
        inside_color = (color / 2).bytes + [127]
        brush = QtGui.QBrush(QtGui.QColor(*inside_color))
        pen = QtGui.QPen(QtGui.QColor(*border_color))

        for v in vertices:
            poly += QtCore.QPointF(*v)

        item = self.scene.addPolygon(poly, brush=brush, pen=pen)
        self.temp_items.append(item)

    def DrawCircleShape(self, shape, transform, color, temporary=False):
        center = b2Mul(transform, shape.pos)
        radius = shape.radius
        axis = transform.R.x_axis

        border_color = color.bytes + [255]
        inside_color = (color / 2).bytes + [127]
        brush = QtGui.QBrush(QtGui.QColor(*inside_color))
        pen = QtGui.QPen(QtGui.QColor(*border_color))
        ellipse = self.scene.addEllipse(-radius, -radius,
                                        radius * 2, radius * 2, brush=brush,
                                        pen=pen)
        line = self.scene.addLine(center[0], center[1],
                                  (center[0] - radius * axis[0]),
                                  (center[1] - radius * axis[1]),
                                  pen=QtGui.QPen(QColor(255, 0, 0)))
        ellipse.setPos(*center)
        ellipse.radius = radius

        if temporary:
            self.temp_items.append(ellipse)
            self.temp_items.append(line)
        else:
            self.item_cache[hash(shape)] = [ellipse, line]

    def DrawPolygonShape(self, shape, transform, color, temporary=False):
        poly = QtGui.QPolygonF()
        border_color = color.bytes + [255]
        inside_color = (color / 2).bytes + [127]
        brush = QtGui.QBrush(QtGui.QColor(*inside_color))
        pen = QtGui.QPen(QtGui.QColor(*border_color))

        for v in shape.vertices:
            poly += QtCore.QPointF(*v)

        item = self.scene.addPolygon(poly, brush=brush, pen=pen)
        item.setRotation(transform.angle * 180.0 / b2_pi)
        item.setPos(*transform.position)

        if temporary:
            self.temp_items.append(item)
        else:
            self.item_cache[hash(shape)] = [item]

    def _remove_from_cache(self, shape):
        items = self.item_cache[hash(shape)]

        del self.item_cache[hash(shape)]
        for item in items:
            self.scene.removeItem(item)

    def DrawShape(self, shape, transform, color, selected=False):
        """
        Draw any type of shape
        """
        cache_hit = False
        if hash(shape) in self.item_cache:
            cache_hit = True
            items = self.item_cache[hash(shape)]
            items[0].setRotation(transform.angle * 180.0 / b2_pi)
            if isinstance(shape, b2CircleShape):
                radius = shape.radius
                if items[0].radius == radius:
                    center = b2Mul(transform, shape.pos)
                    items[0].setPos(*center)
                    line = items[1]
                    axis = transform.R.x_axis
                    line.setLine(center[0], center[1],
                                 (center[0] - radius * axis[0]),
                                 (center[1] - radius * axis[1]))
                else:
                    self._remove_from_cache(shape)
                    cache_hit = False
            else:
                items[0].setPos(*transform.position)

            if not selected or cache_hit:
                return

        if selected:
            color = b2Color(1, 1, 1)
            temporary = True
        else:
            temporary = False

        if isinstance(shape, b2PolygonShape):
            self.DrawPolygonShape(shape, transform, color, temporary)
        elif isinstance(shape, b2EdgeShape):
            v1 = b2Mul(transform, shape.vertex1)
            v2 = b2Mul(transform, shape.vertex2)
            self.DrawSegment(v1, v2, color)
        elif isinstance(shape, b2CircleShape):
            self.DrawCircleShape(shape, transform, color, temporary)
        elif isinstance(shape, b2LoopShape):
            vertices = shape.vertices
            v1 = b2Mul(transform, vertices[-1])
            for v2 in vertices:
                v2 = b2Mul(transform, v2)
                self.DrawSegment(v1, v2, color)
                v1 = v2

    def DrawJoint(self, joint):
        """
        Draw any type of joint
        """
        bodyA, bodyB = joint.bodyA, joint.bodyB
        xf1, xf2 = bodyA.transform, bodyB.transform
        x1, x2 = xf1.position, xf2.position
        p1, p2 = joint.anchorA, joint.anchorB
        color = b2Color(0.5, 0.8, 0.8)

        if isinstance(joint, b2DistanceJoint):
            self.DrawSegment(p1, p2, color)
        elif isinstance(joint, b2PulleyJoint):
            s1, s2 = joint.groundAnchorA, joint.groundAnchorB
            self.DrawSegment(s1, p1, color)
            self.DrawSegment(s2, p2, color)
            self.DrawSegment(s1, s2, color)

        elif isinstance(joint, b2MouseJoint):
            pass  # don't draw it here
        else:
            self.DrawSegment(x1, p1, color)
            self.DrawSegment(p1, p2, color)
            self.DrawSegment(x2, p2, color)

    def ManualDraw(self):
        """
        This implements code normally present in the C++ version, which calls
        the callbacks that you see in this class (DrawSegment, DrawSolidCircle,
        etc.).

        This is implemented in Python as an example of how to do it, and also a
        test.
        """
        colors = {
            'active': b2Color(0.5, 0.5, 0.3),
            'static': b2Color(0.5, 0.9, 0.5),
            'kinematic': b2Color(0.5, 0.5, 0.9),
            'asleep': b2Color(0.6, 0.6, 0.6),
            'default': b2Color(0.9, 0.7, 0.7),
        }

        settings = self.test.settings
        world = self.test.world
        if self.test.selected_shapebody:
            sel_shape, sel_body = self.test.selected_shapebody
        else:
            sel_shape = None

        if settings.drawShapes:
            for body in world.bodies:
                transform = body.transform
                for fixture in body.fixtures:
                    shape = fixture.shape

                    if not body.active:
                        color = colors['active']
                    elif body.type == b2_staticBody:
                        color = colors['static']
                    elif body.type == b2_kinematicBody:
                        color = colors['kinematic']
                    elif not body.awake:
                        color = colors['asleep']
                    else:
                        color = colors['default']

                    self.DrawShape(fixture.shape, transform,
                                   color, (sel_shape == shape))

        if settings.drawJoints:
            for joint in world.joints:
                self.DrawJoint(joint)

        # if settings.drawPairs
        #   pass

        if settings.drawAABBs:
            color = b2Color(0.9, 0.3, 0.9)
            # cm = world.contactManager
            for body in world.bodies:
                if not body.active:
                    continue
                transform = body.transform
                for fixture in body.fixtures:
                    shape = fixture.shape
                    for childIndex in range(shape.childCount):
                        self.DrawAABB(shape.getAABB(
                            transform, childIndex), color)

    def to_screen(self, point):
        """
        In here for compatibility with other frameworks.
        """
        return tuple(point)


class GraphicsScene (QtGui.QGraphicsScene):

    def __init__(self, test, parent=None):
        super(GraphicsScene, self).__init__(parent)
        self.test = test

    def keyPressEvent(self, event):
        self.test._Keyboard_Event(event.key(), down=True)

    def keyReleaseEvent(self, event):
        self.test._Keyboard_Event(event.key(), down=False)

    def mousePressEvent(self, event):
        pos = self.test.ConvertScreenToWorld(
            event.scenePos().x(), event.scenePos().y())
        if event.button() == Qt.RightButton:
            self.test.ShowProperties(pos)
        elif event.button() == Qt.LeftButton:
            if event.modifiers() == Qt.ShiftModifier:
                self.test.ShiftMouseDown(pos)
            else:
                self.test.MouseDown(pos)

    def mouseReleaseEvent(self, event):
        pos = event.scenePos().x(), event.scenePos().y()
        if event.button() == Qt.RightButton:
            self.test.MouseUp(pos)
        elif event.button() == Qt.LeftButton:
            self.test.MouseUp(pos)

    def mouseMoveEvent(self, event):
        pos = event.scenePos().x(), event.scenePos().y()
        self.test.MouseMove(self.test.ConvertScreenToWorld(*pos))
        QtGui.QGraphicsScene.mouseMoveEvent(self, event)


class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self, test, parent=None):
        QtGui.QMainWindow.__init__(self)
        self.setupUi(self)
        self.scene = GraphicsScene(test)
        self.test = test
        self.scene.setBackgroundBrush(QtGui.QBrush(QtGui.QColor(0, 0, 0)))
        self.graphicsView.setScene(self.scene)
        self.graphicsView.scale(self.test.viewZoom, -self.test.viewZoom)
        self.reset_properties_list()
        self.restoreLayout()

        def increase_font_size(amount=1.0):
            self.setFontSize(app.font().pointSize() + amount)

        def decrease_font_size(amount=1.0):
            self.setFontSize(app.font().pointSize() - amount)

        self.mnuExit.triggered.connect(self.close)
        self.mnuIncreaseFontSize.triggered.connect(increase_font_size)
        self.mnuDecreaseFontSize.triggered.connect(decrease_font_size)
        self.add_settings_widgets()

    def add_settings_widgets(self):
        self.settings_widgets = {}

        gb = self.gbOptions  # the options groupbox
        layout = QtGui.QVBoxLayout()
        gb.setLayout(layout)

        for text, variable in settings.checkboxes:
            if variable:
                widget = QtGui.QCheckBox('&' + text)

                def state_changed(value, variable=variable, widget=widget):
                    setattr(self.test.settings, variable, widget.isChecked())

                widget.stateChanged.connect(state_changed)
                widget.setChecked(getattr(self.test.settings, variable))
                self.settings_widgets[variable] = widget
            else:
                widget = QtGui.QLabel(text)
                widget.setAlignment(Qt.AlignHCenter)

            layout.addWidget(widget)

        for slider in settings.sliders:
            label = QtGui.QLabel(slider['text'])
            label.setAlignment(Qt.AlignHCenter)
            layout.addWidget(label)

            widget = QtGui.QScrollBar(Qt.Horizontal)
            widget.setRange(slider['min'], slider['max'])
            var = slider['name']

            def value_changed(value, slider=slider, label=label):
                variable = slider['name']
                text = slider['text']
                setattr(self.test.settings, variable, value)
                label.setText('%s (%d)' % (text, value))

            widget.valueChanged.connect(value_changed)
            self.settings_widgets[var] = widget
            layout.addWidget(widget)

        self.update_widgets_from_settings()

    def update_widgets_from_settings(self, step_settings=None):
        if step_settings is None:
            step_settings = self.test.settings

        for var, widget in list(self.settings_widgets.items()):
            if isinstance(widget, QtGui.QCheckBox):
                widget.setChecked(getattr(step_settings, var))
            else:
                widget.setValue(getattr(step_settings, var))

        for slider in settings.sliders:
            var = slider['name']
            self.settings_widgets[var].setValue(getattr(step_settings, var))

    def reset_properties_list(self):
        self.twProperties.clear()
        self.twProperties.setRowCount(0)
        self.twProperties.setColumnCount(3)
        self.twProperties.verticalHeader().hide()  # don't show numbers on left
        self.twProperties.setHorizontalHeaderLabels(['class', 'name', 'value'])

    def keyPressEvent(self, event):
        self.test._Keyboard_Event(event.key(), down=True)

    def keyReleaseEvent(self, event):
        self.test._Keyboard_Event(event.key(), down=False)

    @property
    def settings(self):
        return QtCore.QSettings("pybox2d", "Framework")

    def setFontSize(self, size):
        """
        Update the global font size
        """
        if size <= 0.0:
            return

        global app
        font = app.font()
        font.setPointSize(size)
        app.setFont(font)

    def restoreLayout(self):
        """
        Restore the layout of each widget
        """
        settings = self.settings
        try:
            self.restoreGeometry(settings.value("geometry").toByteArray())
            self.restoreState(settings.value("windowState").toByteArray())
            size = settings.value('fontSize').toFloat()[0]
            self.setFontSize(size)
        except:
            pass

    def saveLayout(self):
        """
        Save the layout of each widget
        """
        settings = self.settings
        settings.setValue("geometry", self.saveGeometry())
        settings.setValue("windowState", self.saveState())
        settings.setValue("fontSize", app.font().pointSize())

    def closeEvent(self, event):
        QtGui.QMainWindow.closeEvent(self, event)
        self.saveLayout()

app = None


class Pyqt4Framework(FrameworkBase):
    TEXTLINE_START = 0

    def setup_keys(self):
        # Only basic keys are mapped for now: K_[a-z0-9], K_F[1-12] and
        # K_COMMA.

        for letter in string.ascii_uppercase:
            setattr(Keys, 'K_' + letter.lower(),
                    getattr(Qt, 'Key_%s' % letter))
        for i in range(0, 10):
            setattr(Keys, 'K_%d' % i, getattr(Qt, 'Key_%d' % i))
        for i in range(1, 13):
            setattr(Keys, 'K_F%d' % i, getattr(Qt, 'Key_F%d' % i))
        Keys.K_LEFT = Qt.Key_Left
        Keys.K_RIGHT = Qt.Key_Right
        Keys.K_UP = Qt.Key_Up
        Keys.K_DOWN = Qt.Key_Down
        Keys.K_HOME = Qt.Key_Home
        Keys.K_PAGEUP = Qt.Key_PageUp
        Keys.K_PAGEDOWN = Qt.Key_PageDown
        Keys.K_COMMA = Qt.Key_Comma
        Keys.K_SPACE = Qt.Key_Space

    def __reset(self):
        # Screen/rendering-related
        self._viewZoom = 10.0
        self._viewCenter = None
        self._viewOffset = None
        self.screenSize = None
        self.textLine = 0
        self.font = None
        self.fps = 0
        self.selected_shapebody = None, None

        # GUI-related
        self.window = None
        self.setup_keys()

    def __init__(self):
        super(Pyqt4Framework, self).__init__()

        self.__reset()

        if settings.fwSettings.onlyInit:  # testing mode doesn't initialize Pyqt4
            return

        global app
        app = QtGui.QApplication(sys.argv)

        print('Initializing Pyqt4 framework...')

        # Pyqt4 Initialization
        self.window = MainWindow(self)
        self.window.show()

        self.window.setWindowTitle("Python Box2D Testbed - " + self.name)
        self.renderer = Pyqt4Draw(self)

        # Note that in this framework, we override the draw debug data routine
        # that occurs in Step(), and we implement the normal C++ code in
        # Python.
        self.world.DrawDebugData = lambda: self.renderer.ManualDraw()
        self.screenSize = b2Vec2(0, 0)
        self.viewCenter = (0, 10.0 * 20.0)
        self.groundbody = self.world.CreateBody()

    def setCenter(self, value):
        """
        Updates the view offset based on the center of the screen.

        Tells the debug draw to update its values also.
        """
        self._viewCenter = b2Vec2(*value)
        self._viewOffset = self._viewCenter - self.screenSize / 2
        self.window.graphicsView.centerOn(*self._viewCenter)

    def setZoom(self, zoom):
        self._viewZoom = zoom
        self.window.graphicsView.resetTransform()
        self.window.graphicsView.scale(self._viewZoom, -self._viewZoom)
        self.window.graphicsView.centerOn(*self._viewCenter)

    viewZoom = property(lambda self: self._viewZoom, setZoom,
                        doc='Zoom factor for the display')
    viewCenter = property(lambda self: self._viewCenter, setCenter,
                          doc='Screen center in camera coordinates')
    viewOffset = property(lambda self: self._viewOffset,
                          doc='The offset of the top-left corner of the screen')

    def run(self):
        """
        What would be the main loop is instead a call to
        app.exec_() for the event-driven pyqt4.
        """
        global app
        self.step_timer = QtCore.QTimer()

        self.step_timer.timeout.connect(self.SimulationLoop)
        self.window.twProperties.itemChanged.connect(self.prop_cell_changed)
        self.step_timer.start(int((1000.0 / self.settings.hz)))

        app.exec_()

        self.step_timer.stop()
        print('Cleaning up...')
        self.world.contactListener = None
        self.world.destructionListener = None
        self.world.renderer = None
        self.world = None

    def _Keyboard_Event(self, key, down=True):
        """
        Internal keyboard event, don't override this.

        Checks for the initial keydown of the basic testbed keys. Passes the unused
        ones onto the test via the Keyboard() function.
        """
        if down:
            if key == Keys.K_z:       # Zoom in
                self.viewZoom = min(1.10 * self.viewZoom, 50.0)
            elif key == Keys.K_x:     # Zoom out
                self.viewZoom = max(0.9 * self.viewZoom, 0.02)
            elif key == Keys.K_SPACE:  # Launch a bomb
                self.LaunchRandomBomb()
            else:              # Inform the test of the key press
                self.Keyboard(key)
        else:
            self.KeyboardUp(key)

    def CheckKeys(self):
        pass

    def _ShowProperties(self, obj):
        self.selected_shapebody = None, None

        class_ = obj.__class__
        ignore_list = ('thisown',)

        i = 0
        twProperties = self.window.twProperties
        # Get all of the members of the class
        for prop in dir(class_):
            # If they're properties and not to be ignored, add them to the
            # table widget
            if (isinstance(getattr(class_, prop), property)
                    and prop not in ignore_list):
                try:
                    value = getattr(obj, prop)
                except:
                    # Write-only?
                    continue

                widget = None

                # Attempt to determine whether it's read-only or not
                try:
                    setattr(obj, prop, value)
                except:
                    editable = False
                else:
                    editable = True

                # Increase the row count and insert the new item
                twProperties.setRowCount(twProperties.rowCount() + 1)
                i = twProperties.rowCount() - 1
                self.item = QTableWidgetItem(class_.__name__)
                twProperties.setItem(i, 0, QTableWidgetItem(
                    class_.__name__))  # class name
                twProperties.item(i, 0).setFlags(Qt.ItemIsEnabled)

                twProperties.setItem(
                    i, 1, QtGui.QTableWidgetItem(prop))      # prop name
                twProperties.item(i, 1).setFlags(Qt.ItemIsEnabled)

                # and finally, the property values
                # booleans are checkboxes
                if isinstance(value, bool):
                    def state_changed(value, prop=prop):
                        self.property_changed(prop, value == Qt.Checked)

                    widget = QtGui.QCheckBox('')
                    widget.stateChanged.connect(state_changed)
                    if value:
                        widget.setCheckState(Qt.Checked)

                # ints, floats are spinboxes
                elif isinstance(value, (int, float)):
                    def value_changed(value, prop=prop):
                        self.property_changed(prop, value)

                    widget = QtGui.QDoubleSpinBox()
                    widget.valueChanged.connect(value_changed)
                    widget.setValue(value)
                # lists turn into -- lists
                elif isinstance(value, list):
                    widget = QtGui.QListWidget()
                    for entry in value:
                        widget.addItem(str(entry))
                    if value:
                        # sz=widget.item(0).sizeHint()
                        # print(sz, sz.width(), sz.height())
                        # sz.setHeight(sz.height()*2)
                        # widget.setMinimumSize(sz)
                        # widget.setMinimumSize(QtCore.QSize(1,60))
                        pass  # TODO
                # vec2s will be shown as a textbox
                elif isinstance(value, b2Vec2):
                    value = '(%.2f, %.2f)' % (tuple(value))
                else:
                    pass

                if widget:
                    twProperties.setCellWidget(i, 2, widget)
                    if hasattr(widget, 'setReadOnly'):
                        widget.setReadOnly(not editable)
                    elif hasattr(widget, 'setEnabled'):
                        widget.setEnabled(editable)
                else:
                    # Just using the table widget, set the cell text
                    cell = QtGui.QTableWidgetItem(str(value))
                    if editable:
                        cell.setFlags(Qt.ItemIsEditable | Qt.ItemIsEnabled)
                    else:
                        cell.setFlags(Qt.ItemIsEnabled)
                    twProperties.setItem(i, 2, cell)

                i += 1

    # callback indicating a cell in the table widget was changed
    def prop_cell_changed(self, twi):
        if twi.column() != 2:  # the data column
            return

        row = twi.row()
        prop = str(self.window.twProperties.item(row, 1).text())
        self.property_changed(prop, str(twi.text()))

    # callback indicating one of the property widgets was modified
    def property_changed(self, prop, value=None):
        if not self.selected_shapebody[0]:
            return

        print('Trying to change %s to %s...' % (prop, value))
        shape, body = self.selected_shapebody
        for inst in (shape, body):
            if hasattr(inst, prop):
                try:
                    cur_value = getattr(inst, prop)
                    if isinstance(cur_value, b2Vec2):
                        m = re.search('\(?([\d\.]*)\s*,\s*([\d\.]*)\)?', value)
                        if m:
                            x, y = m.groups()
                            value = (float(x), float(y))
                except:
                    raise
                    pass

                try:
                    setattr(inst, prop, value)
                except:
                    print('Failed - %s' % sys.exc_info()[1])

    def ShowProperties(self, p):
        aabb = b2AABB(lowerBound=p - (0.001, 0.001),
                      upperBound=p + (0.001, 0.001))

        # Query the world for overlapping shapes.
        query = fwQueryCallback(p)
        self.world.QueryAABB(query, aabb)

        if query.fixture:
            self.window.reset_properties_list()

            fixture = query.fixture
            body = fixture.body
            self._ShowProperties(body)

            shape = fixture.shape
            self._ShowProperties(shape)

            self.selected_shapebody = (shape, body)

    def Step(self, settings):
        super(Pyqt4Framework, self).Step(settings)

    def ConvertScreenToWorld(self, x, y):
        """
        PyQt4 gives us transformed positions, so no need to convert
        """
        return b2Vec2(x, y)

    DrawStringAt = lambda self, *args: self.renderer.DrawStringAt(*args)

    def Print(self, str, color=(229, 153, 153, 255)):
        """
        Draw some text at the top status lines and advance to the next line.
        """
        self.DrawStringAt(5, self.textLine, str, color)
        self.textLine += self.renderer.font_spacing

    def Keyboard(self, key):
        """
        Callback indicating 'key' has been pressed down.
        The keys are mapped after pygame's style.

         from framework import Keys
         if key == Keys.K_z:
             ...
        """
        pass

    def KeyboardUp(self, key):
        """
        Callback indicating 'key' has been released.
        See Keyboard() for key information
        """
        pass

    def FixtureDestroyed(self, fixture):
        shape = fixture.shape
        if shape == self.selected_shapebody[0]:
            self.selected_shapebody = None, None
            self.window.reset_properties_list()
        if hash(shape) in self.renderer.item_cache:
            scene_items = self.renderer.item_cache[hash(shape)]
            for item in scene_items:
                self.window.scene.removeItem(item)
            del self.renderer.item_cache[hash(shape)]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from Box2D import *
import Box2D

class cl (b2ContactListener):
    pass

class test_body (unittest.TestCase):
    def setUp(self):
        pass

    def test_world(self):
        world = b2World(gravity=(0,-10), doSleep=True)
        world = b2World((0,-10), True)
        world = b2World((0,-10), doSleep=True)

    def test_extended(self):
        world = b2World()
        fixture1=b2FixtureDef(shape=b2CircleShape(radius=1), density=1, friction=0.3)
        fixture2=b2FixtureDef(shape=b2CircleShape(radius=2), density=1, friction=0.3)
        shape1=b2PolygonShape(box=(5,1))
        shape2=b2PolygonShape(box=(5,1))
        shapefixture=b2FixtureDef(density=2.0, friction=0.3)

        world.CreateStaticBody(fixtures=[fixture1, fixture2],
                shapes=[shape1, shape2], shapeFixture=shapefixture)

        # make sure that 4 bodies were created
        self.assertEqual(len(world.bodies[-1].fixtures), 4)

        world.CreateKinematicBody(fixtures=[fixture1, fixture2],
                shapes=[shape1, shape2], shapeFixture=shapefixture)
        self.assertEqual(len(world.bodies[-1].fixtures), 4)
        world.CreateDynamicBody(fixtures=[fixture1, fixture2],
                shapes=[shape1, shape2], shapeFixture=shapefixture)
        self.assertEqual(len(world.bodies[-1].fixtures), 4)

    def test_body(self):
        self.cont_list=cl()
        world = b2World(gravity=(0,-10), doSleep=True, contactListener=self.cont_list)
        groundBody = world.CreateBody(b2BodyDef(position=(0,-10)))

        groundBody.CreateFixturesFromShapes(shapes=b2PolygonShape(box=(50,10)))

        body = world.CreateBody(b2BodyDef(type=b2_dynamicBody, position=(0,4)))

        body.CreateFixture(b2FixtureDef(shape=b2CircleShape(radius=1), density=1, friction=0.3))

        timeStep = 1.0 / 60
        vel_iters, pos_iters = 6, 2

        for i in range(60):
            world.Step(timeStep, vel_iters, pos_iters)
            world.ClearForces()

    def test_new_createfixture(self):
        world = b2World(gravity=(0,-10), doSleep=True)
        body=world.CreateDynamicBody(position=(0,0))
        body.CreateCircleFixture(radius=0.2, friction=0.2, density=1.0)
        body.fixtures[0]
        body.fixtures[0].friction
        body.fixtures[0].density
        body.fixtures[0].shape.radius

        body.CreatePolygonFixture(box=(1,1), friction=0.2, density=1.0)
        body.fixtures[1]
        body.fixtures[1].friction
        body.fixtures[1].density
        body.fixtures[1].shape.vertices

        v1=(-10, 0)
        v2=(-7, -1)
        v3=(-4, 0)
        v4=(0, 0)
        body.CreateEdgeFixture(vertices=[v1,v2,v3,v4], friction=0.3, density=1.0)
        body.fixtures[2]
        body.fixtures[2].friction
        body.fixtures[2].density
        body.fixtures[2].shape.vertices

        #TODO Loop shapes

    def test_fixture_without_shape(self):
        world = b2World(gravity=(0,-10), doSleep=True)
        body = world.CreateDynamicBody(position=(0,0))

        self.assertRaises(ValueError, body.CreateFixture)


if __name__ == '__main__':
    unittest.main()




"""Contains various types of button widgets."""

from pygame.locals import *

from .const import *
from . import widget, surface
from . import basic

class _button(widget.Widget):
    # The underlying 'value' accessed by the getter and setters below
    _value = None

    def __init__(self,**params):
        widget.Widget.__init__(self,**params)
        self.state = 0
    
    def event(self,e):
        if e.type == ENTER: self.repaint()
        elif e.type == EXIT: self.repaint()
        elif e.type == FOCUS: self.repaint()
        elif e.type == BLUR: self.repaint()
        elif e.type == KEYDOWN:
            if e.key == K_SPACE or e.key == K_RETURN:
                self.state = 1
                self.repaint()
        elif e.type == MOUSEBUTTONDOWN: 
            self.state = 1
            self.repaint()
        elif e.type == KEYUP:
            if self.state == 1:
                sub = pygame.event.Event(CLICK,{'pos':(0,0),'button':1})
                #self.send(sub.type,sub)
                self._event(sub)
            
            self.state = 0
            self.repaint()
        elif e.type == MOUSEBUTTONUP:
            self.state = 0
            self.repaint()
        elif e.type == CLICK:
            self.click()
        
        self.pcls = ""
        if self.state == 0 and self.is_hovering():
            self.pcls = "hover"
        if self.state == 1 and self.is_hovering():
            self.pcls = "down"
    
    def click(self): 
        pass


class Button(_button):
    """A button, buttons can be clicked, they are usually used to set up callbacks.
    
    Example:
        w = gui.Button("Click Me")
        w.connect(gui.CLICK, fnc, value)

    """

    def __init__(self, value=None, **params):
        """Button constructor, which takes either a string label or widget.
        
        See Widget documentation for additional style parameters.

        """
        params.setdefault('cls', 'button')
        _button.__init__(self, **params)
        self.value = value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, val):
        if (isinstance(val, basestring)):
            # Allow the choice of font to propagate to the button label
            params = {}
            if (self.style.font):
                params["font"] = self.style.font
            val = basic.Label(val, cls=self.cls+".label", **params)
            val.container = self

        oldval = self._value
        self._value = val

        if (val != oldval):
            self.send(CHANGE)
            self.chsize()

    def resize(self,width=None,height=None):
        self.value.rect.x,self.value.rect.y = 0,0
        self.value.rect.w,self.value.rect.h = self.value.resize(width,height)
        return self.value.rect.w,self.value.rect.h

    def paint(self,s):
        self.value.pcls = self.pcls
        self.value.paint(surface.subsurface(s,self.value.rect))


class Switch(_button):
    """A switch can have two states, on or off."""

    def __init__(self,value=False,**params):
        params.setdefault('cls','switch')
        _button.__init__(self,**params)
        self.value = value
        
        img = self.style.off
        self.style.width = img.get_width()
        self.style.height = img.get_height()
    
    def paint(self,s):
        #self.pcls = ""
        #if self.container.myhover is self: self.pcls = "hover"
        if self.value: img = self.style.on
        else: img = self.style.off
        s.blit(img,(0,0))
    
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, val):
        oldval = self._value
        self._value = val
        if oldval != val:
            self.send(CHANGE)
            self.repaint()
    
    def click(self):
        self.value = not self.value

class Checkbox(_button):    
    """A type of switch that can be grouped with other checkboxes.
    
    Example:
        # The 'value' parameter indicates which checkboxes are on by default
        g = gui.Group(name='colors',value=['r','b'])
        
        t = gui.Table()
        t.tr()
        t.td(gui.Label('Red'))
        t.td(gui.Checkbox(g,'r'))
        t.tr()
        t.td(gui.Label('Green'))
        t.td(gui.Checkbox(g,'g'))
        t.tr()
        t.td(gui.Label('Blue'))
        t.td(gui.Checkbox(g,'b'))

    """
    
    def __init__(self,group,value=None,**params):
        """Checkbox constructor.

        Keyword arguments:
            group -- the Group that this checkbox belongs to
            value -- the initial value (True or False)
    
        See Widget documentation for additional style parameters.

        """

        params.setdefault('cls','checkbox')
        _button.__init__(self,**params)
        self.group = group
        self.group.add(self)
        if self.group.value == None:
            self.group.value = []
        self.value = value
        
        img = self.style.off
        self.style.width = img.get_width()
        self.style.height = img.get_height()
    
    def paint(self,s):
        #self.pcls = ""
        #if self.container.myhover is self: self.pcls = "hover"
        if self.value in self.group.value: img = self.style.on
        else: img = self.style.off
        
        s.blit(img,(0,0))
    
    def click(self):
        if self.value in self.group.value:
            self.group.value.remove(self.value)
        else:
            self.group.value.append(self.value)
        self.group._change()

class Radio(_button):
    """A type of switch that can be grouped with other radio buttons, except
    that only one radio button can be active at a time.
    
    Example:
        g = gui.Group(name='colors',value='g')
        
        t = gui.Table()
        t.tr()
        t.td(gui.Label('Red'))
        t.td(gui.Radio(g,'r'))
        t.tr()
        t.td(gui.Label('Green'))
        t.td(gui.Radio(g,'g'))
        t.tr()
        t.td(gui.Label('Blue'))
        t.td(gui.Radio(g,'b'))

    """    
    
    def __init__(self,group=None,value=None,**params):
        """Radio constructor.

        Keyword arguments:    
            group -- the Group this radio button belongs to
            value -- the initial value (True or False)

        """
        params.setdefault('cls','radio')
        _button.__init__(self,**params)
        self.group = group
        self.group.add(self)
        self.value = value
        
        img = self.style.off
        self.style.width = img.get_width()
        self.style.height = img.get_height()
    
    def paint(self,s):
        #self.pcls = ""
        #if self.container.myhover is self: self.pcls = "hover"
        if self.group.value == self.value: img = self.style.on
        else: img = self.style.off
        s.blit(img,(0,0))
    
    def click(self):
        self.group.value = self.value

class Tool(_button):
    """Within a Group of Tool widgets only one may be selected at a time.

    Example:
        g = gui.Group(name='colors',value='g')
        
        t = gui.Table()
        t.tr()
        t.td(gui.Tool(g,'Red','r'))
        t.tr()
        t.td(gui.Tool(g,'Green','g'))
        t.tr()
        t.td(gui.Tool(g,'Blue','b'))

    """

    def __init__(self,group,widget=None,value=None,**params): #TODO widget= could conflict with module widget
        """Tool constructor.

        Keyword arguments:    
            group -- a gui.Group for the Tool to belong to
            widget -- a widget to appear on the Tool (similar to a Button)
            value -- the value

        """
        params.setdefault('cls','tool')
        _button.__init__(self,**params)
        self.group = group
        self.group.add(self)
        self.value = value
        
        if widget:
            self.setwidget(widget)
        
        if self.group.value == self.value: self.pcls = "down"
    
    def setwidget(self,w):
        self.widget = w
    
    def resize(self,width=None,height=None):
        self.widget.rect.w,self.widget.rect.h = self.widget.resize()
        #self.widget._resize()
        #self.rect.w,self.rect.h = self.widget.rect_margin.w,self.widget.rect_margin.h
        
        return self.widget.rect.w,self.widget.rect.h
    
    def event(self,e):
        _button.event(self,e)
        if self.group.value == self.value: self.pcls = "down"
    
    def paint(self,s):
        if self.group.value == self.value: self.pcls = "down"
        self.widget.paint(surface.subsurface(s,self.widget.rect))
    
    def click(self):
        self.group.value = self.value
        for w in self.group.widgets:
            if w != self: w.pcls = ""

            
class Icon(_button):
    """TODO - might be deprecated
    """
    def __init__(self,cls,**params):
        params['cls'] = cls
        _button.__init__(self,**params)
        s = self.style.image
        self.style.width = s.get_width()
        self.style.height = s.get_height()
        self.state = 0
    
    def paint(self,s):
        #self.pcls = ""
        #if self.state == 0 and hasattr(self.container,'myhover') and self.container.myhover is self: self.pcls = "hover"
        #if self.state == 1 and hasattr(self.container,'myhover') and self.container.myhover is self: self.pcls = "down"
        s.blit(self.style.image,(0,0))

class Link(_button):
    """A link, links can be clicked, they are usually used to set up callbacks.
    Basically the same as the button widget, just text only with a different cls.
    Made for convenience.
    
    Example:
        w = gui.Link("Click Me")
        w.connect(gui.CLICK,fnc,value)

    """
    def __init__(self,value,**params):
        params.setdefault('focusable',True)
        params.setdefault('cls','link')
        _button.__init__(self,**params)
        self.value = value
        self.font = self.style.font
        self.style.width, self.style.height = self.font.size(self.value)
    
    def paint(self,s):
        s.blit(self.font.render(self.value, 1, self.style.color),(0,0))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

try:
    from ..pgu import gui
except:
    raise ImportError('Unable to load PGU')

from ..settings import checkboxes
from ..settings import sliders


class fwGUI(gui.Table):
    """
    Deals with the initialization and changing the settings based on the GUI
    controls. Callbacks are not used, but the checkboxes and sliders are polled
    by the main loop.
    """
    form = None

    def __init__(self,settings, **params):
        # The framework GUI is just basically a HTML-like table
        # There are 2 columns right-aligned on the screen
        gui.Table.__init__(self,**params)
        self.form=gui.Form()

        fg = (255,255,255)

        # "Toggle menu"
        self.tr()
        self.td(gui.Label("F1: Toggle Menu",color=(255,0,0)),align=1,colspan=2)

        for slider in sliders:
            # "Slider title"
            self.tr()
            self.td(gui.Label(slider['text'],color=fg),align=1,colspan=2)

            # Create the slider
            self.tr()
            e = gui.HSlider(getattr(settings, slider['name']),slider['min'],slider['max'],size=20,width=100,height=16,name=slider['name'])
            self.td(e,colspan=2,align=1)

        # Add each of the checkboxes.
        for text, variable in checkboxes:
            self.tr()
            if variable == None:
                # Checkboxes that have no variable (i.e., None) are just labels.
                self.td(gui.Label(text, color=fg), align=1, colspan=2)
            else:
                # Add the label and then the switch/checkbox
                self.td(gui.Label(text, color=fg), align=1)
                self.td(gui.Switch(value=getattr(settings, variable),name=variable))

    def updateGUI(self, settings):
        """
        Change all of the GUI elements based on the current settings
        """
        for text, variable in checkboxes:
            if not variable: continue
            if hasattr(settings, variable):
                self.form[variable].value = getattr(settings, variable)

        # Now do the sliders
        for slider in sliders:
            name=slider['name']
            self.form[name].value=getattr(settings, name)

    def updateSettings(self, settings):
        """
        Change all of the settings based on the current state of the GUI.
        """
        for text, variable in checkboxes:
            if variable:
                setattr(settings, variable, self.form[variable].value)

        # Now do the sliders
        for slider in sliders:
            name=slider['name']
            setattr(settings, name, int(self.form[name].value))

        # If we're in single-step mode, update the GUI to reflect that.
        if settings.singleStep:
            settings.pause=True
            self.form['pause'].value = True
            self.form['singleStep'].value = False




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from Box2D import *

class cl (b2ContactListener):
    pass

class test_kwargs (unittest.TestCase):
    def setUp(self):
        pass

    def test_kwargs(self):
        self.cont_list=cl()
        world = b2World(gravity=(0,-10), doSleep=True, contactListener=self.cont_list)
        groundBody = world.CreateBody(b2BodyDef(position=(0,-10)))

        groundBody.CreateFixturesFromShapes(b2PolygonShape(box=(50,10)))

        body = world.CreateBody(b2BodyDef(type=b2_dynamicBody, position=(0,4),
                        fixtures=[]))

        body = world.CreateBody(
                type=b2_dynamicBody,
                position=(0,4),
                fixtures=b2FixtureDef(shape=b2PolygonShape(box=(2,1)), density=1.0)
                )

        body = world.CreateBody(
                    type=b2_dynamicBody,
                    position=(0,4),
                    shapes=(b2PolygonShape(box=(2,1)), b2PolygonShape(box=(2,1))),
                    shapeFixture=b2FixtureDef(density=1.0),
                )

        body = world.CreateBody(
                    type=b2_dynamicBody,
                    position=(0,4),
                    fixtures=b2FixtureDef(shape=b2CircleShape(radius=1), density=1, friction=0.3),
                    shapes=(b2PolygonShape(box=(2,1)), b2PolygonShape(box=(2,1))),
                    shapeFixture=b2FixtureDef(density=1.0),
                )

        body.CreateFixture(shape=b2CircleShape(radius=1), density=1, friction=0.3)
        timeStep = 1.0 / 60
        vel_iters, pos_iters = 6, 2

        for i in range(60):
            world.Step(timeStep, vel_iters, pos_iters)
            world.ClearForces()

    def test_body(self):
        world = b2World(gravity=(0,-10), doSleep=True)
        body = world.CreateBody(b2BodyDef())
        body2 = world.CreateBody(position=(1,1))

    def test_joints(self):
        world = b2World(gravity=(0,-10), doSleep=True)
        body = world.CreateBody(b2BodyDef())
        body2 = world.CreateBody(position=(1,1))
        world.CreateJoint(type=b2RevoluteJoint, bodyA=body, bodyB=body2)
        world.CreateJoint(type=b2RevoluteJointDef, bodyA=body, bodyB=body2)

        kwargs=dict(type=b2RevoluteJointDef, bodyA=body, bodyB=body2)
        world.CreateJoint(**kwargs)

if __name__ == '__main__':
    unittest.main()




"""
"""
import pygame
from pygame.locals import *

from .const import *
from . import widget, surface
from . import pguglobals

class Container(widget.Widget):
    """The base container widget, can be used as a template as well as stand alone."""

    def __init__(self,**params):
        widget.Widget.__init__(self,**params)
        self.myfocus = None
        self.mywindow = None
        self.myhover = None
        #self.background = 0
        self.widgets = []
        self.windows = []
        self.toupdate = {}
        self.topaint = {}
    
    def update(self,s):
        updates = []
        
        if self.myfocus: self.toupdate[self.myfocus] = self.myfocus
        
        for w in self.topaint:
            if w is self.mywindow:
                continue
            else:
                sub = surface.subsurface(s,w.rect)
                #if (hasattr(w, "_container_bkgr")):
                #    sub.blit(w._container_bkgr,(0,0))
                w.paint(sub)
                updates.append(pygame.rect.Rect(w.rect))
        
        for w in self.toupdate:
            if w is self.mywindow:
                continue
            else:            
                us = w.update(surface.subsurface(s,w.rect))
            if us:
                for u in us:
                    updates.append(pygame.rect.Rect(u.x + w.rect.x,u.y+w.rect.y,u.w,u.h))
        
        for w in self.topaint:
            if w is self.mywindow:
                w.paint(self.top_surface(s,w))
                updates.append(pygame.rect.Rect(w.rect))
            else:
                continue 
        
        for w in self.toupdate:
            if w is self.mywindow:
                us = w.update(self.top_surface(s,w))
            else:            
                continue 
            if us:
                for u in us:
                    updates.append(pygame.rect.Rect(u.x + w.rect.x,u.y+w.rect.y,u.w,u.h))
        
        self.topaint = {}
        self.toupdate = {}
        
        return updates
    
    def repaint(self,w=None):
        if not w:
            return widget.Widget.repaint(self)
        self.topaint[w] = w
        self.reupdate()
    
    def reupdate(self,w=None):
        if not w:
            return widget.Widget.reupdate(self)
        self.toupdate[w] = w
        self.reupdate()
    
    def paint(self,s):
        self.toupdate = {}
        self.topaint = {}
        for w in self.widgets:
            try:
                sub = surface.subsurface(s, w.rect)
            except: 
                print('container.paint(): %s not inside %s' % (
                    w.__class__.__name__,self.__class__.__name__))
                print(s.get_width(), s.get_height(), w.rect)
                print("")
            else:
#                if (not hasattr(w,'_container_bkgr') or 
#                    w._container_bkgr.get_size() != sub.get_size()):
#                         #w._container_bkgr.get_width() == sub.get_width() and 
#                         #w._container_bkgr.get_height() == sub.get_height())):
#                    w._container_bkgr = sub.copy()
#                w._container_bkgr.fill((0,0,0,0))
#                w._container_bkgr.blit(sub,(0,0))
                w.paint(sub)

        for w in self.windows:
            w.paint(self.top_surface(s,w))
    
    def top_surface(self,s,w):
        x,y = s.get_abs_offset()
        s = s.get_abs_parent()
        return surface.subsurface(s,(x+w.rect.x,y+w.rect.y,w.rect.w,w.rect.h))
    
    def event(self,e):
        used = False
        
        if self.mywindow and e.type == MOUSEBUTTONDOWN:
            w = self.mywindow
            if self.myfocus is w:
                if not w.collidepoint(e.pos): self.blur(w)
            if not self.myfocus:
                if w.collidepoint(e.pos): self.focus(w)
        
        if not self.mywindow:
            #### by Gal Koren
            ##
            ## if e.type == FOCUS:
            if e.type == FOCUS and not self.myfocus:
                #self.first()
                pass
            elif e.type == EXIT:
                if self.myhover: self.exit(self.myhover)
            elif e.type == BLUR:
                if self.myfocus: self.blur(self.myfocus)
            elif e.type == MOUSEBUTTONDOWN:
                h = None
                for w in self.widgets:
                    if not w.disabled: 
                        # Focusable not considered, since that is only for tabs
                        if w.collidepoint(e.pos):
                            h = w
                            if self.myfocus is not w: self.focus(w)
                if not h and self.myfocus:
                    self.blur(self.myfocus)
            elif e.type == MOUSEMOTION:
                if 1 in e.buttons:
                    if self.myfocus: ws = [self.myfocus]
                    else: ws = []
                else: ws = self.widgets
                
                h = None
                for w in ws:
                    if w.collidepoint(e.pos):
                        h = w
                        if self.myhover is not w: 
                            self.enter(w)
                        break
                if not h and self.myhover:
                    self.exit(self.myhover)
                w = self.myhover
                
                if w and w is not self.myfocus:
                    sub = pygame.event.Event(e.type,{
                        'buttons':e.buttons,
                        'pos':(e.pos[0]-w.rect.x,e.pos[1]-w.rect.y),
                        'rel':e.rel})
                    used = w._event(sub)
        
        w = self.myfocus
        if w:
            if e.type == MOUSEBUTTONUP or e.type == MOUSEBUTTONDOWN:
                sub = pygame.event.Event(e.type,{
                    'button':e.button,
                    'pos':(e.pos[0]-w.rect.x,e.pos[1]-w.rect.y)})
            elif e.type == CLICK and self.myhover is w:
                sub = pygame.event.Event(e.type,{
                    'button':e.button,
                    'pos':(e.pos[0]-w.rect.x,e.pos[1]-w.rect.y)})
            elif e.type == MOUSEMOTION:
                sub = pygame.event.Event(e.type,{
                    'buttons':e.buttons,
                    'pos':(e.pos[0]-w.rect.x,e.pos[1]-w.rect.y),
                    'rel':e.rel})
            elif (e.type == KEYDOWN or e.type == KEYUP):
                sub = e
            else:
                sub = None

            #elif e.type == CLICK: #a dead click
            #    sub = None

            if (sub):
                used = w._event(sub)

        if not used and e.type is KEYDOWN:
            if e.key is K_TAB and self.myfocus:
                if (e.mod&KMOD_SHIFT) == 0:
                    self.myfocus.next()
                else:
                    self.myfocus.previous()
                    return True
            elif e.key == K_UP: 
                self._move_focus(0,-1)
                return True
            elif e.key == K_RIGHT:
                self._move_focus(1,0)
                return True
            elif e.key == K_DOWN:
                self._move_focus(0,1)
                return True
            elif e.key == K_LEFT:
                self._move_focus(-1,0)
                return True
        return used

    def _move_focus(self,dx_,dy_):
        myfocus = self.myfocus
        if not self.myfocus: return
        
        widgets = self._get_widgets(pguglobals.app)
        #if myfocus not in widgets: return
        #widgets.remove(myfocus)
        if myfocus in widgets:
            widgets.remove(myfocus)
        rect = myfocus.get_abs_rect()
        fx,fy = rect.centerx,rect.centery
        
        def sign(v):
            if v < 0: return -1
            if v > 0: return 1
            return 0
        
        dist = []
        for w in widgets:
            wrect = w.get_abs_rect()
            wx,wy = wrect.centerx,wrect.centery
            dx,dy = wx-fx,wy-fy
            if dx_ > 0 and wrect.left < rect.right: continue
            if dx_ < 0 and wrect.right > rect.left: continue
            if dy_ > 0 and wrect.top < rect.bottom: continue
            if dy_ < 0 and wrect.bottom > rect.top: continue
            dist.append((dx*dx+dy*dy,w))
        if not len(dist): return
        dist.sort()
        d,w = dist.pop(0)
        w.focus()
        
    def _get_widgets(self,c):
        widgets = []
        if c.mywindow:
            widgets.extend(self._get_widgets(c.mywindow))
        else:
            for w in c.widgets:
                if isinstance(w,Container):
                    widgets.extend(self._get_widgets(w))
                elif not w.disabled and w.focusable:
                    widgets.append(w)
        return widgets

    def remove(self,w):
        """Remove a widget from the container."""
        self.blur(w)
        self.widgets.remove(w)
        #self.repaint()
        self.chsize()
    
    def add(self,w,x,y):
        """Add a widget to the container given the position."""
        w.style.x = x
        w.style.y = y 
        w.container = self
        #NOTE: this might fix it, sort of...
        #but the thing is, we don't really want to resize
        #something if it is going to get resized again later
        #for no reason...
        #w.rect.x,w.rect.y = w.style.x,w.style.y
        #w.rect.w, w.rect.h = w.resize()
        self.widgets.append(w)
        self.chsize()
    
    def open(self,w=None,x=None,y=None):
        if (not w):
            w = self

        if (x != None):
            # The position is relative to this container
            rect = self.get_abs_rect()
            pos = (rect.x + x, rect.y + y)
        else:
            pos = None
        # Have the application open the window
        pguglobals.app.open(w, pos)
        
    def focus(self,w=None):
        widget.Widget.focus(self) ### by Gal koren
#        if not w:
#            return widget.Widget.focus(self)
        if not w: return
        if self.myfocus: self.blur(self.myfocus)
        if self.myhover is not w: self.enter(w)
        self.myfocus = w
        w._event(pygame.event.Event(FOCUS))
        
        #print self.myfocus,self.myfocus.__class__.__name__
    
    def blur(self,w=None):
        if not w:
            return widget.Widget.blur(self)
        if self.myfocus is w:
            if self.myhover is w: self.exit(w)
            self.myfocus = None
            w._event(pygame.event.Event(BLUR))
    
    def enter(self,w):
        if self.myhover: self.exit(self.myhover)
        self.myhover = w
        w._event(pygame.event.Event(ENTER))
    
    def exit(self,w):
        if self.myhover and self.myhover is w:
            self.myhover = None
            w._event(pygame.event.Event(EXIT))    
    
    
#     def first(self):
#         for w in self.widgets:
#             if w.focusable:
#                 self.focus(w)
#                 return
#         if self.container: self.container.next(self)
    
#     def next(self,w):
#         if w not in self.widgets: return #HACK: maybe.  this happens in windows for some reason...
#         
#         for w in self.widgets[self.widgets.index(w)+1:]:
#             if w.focusable:
#                 self.focus(w)
#                 return
#         if self.container: return self.container.next(self)
    
    
    def _next(self,orig=None):
        start = 0
        if orig in self.widgets: start = self.widgets.index(orig)+1
        for w in self.widgets[start:]:
            if not w.disabled and w.focusable:
                if isinstance(w,Container):
                    if w._next():
                        return True
                else:
                    self.focus(w)
                    return True
        return False
    
    def _previous(self,orig=None):
        end = len(self.widgets)
        if orig in self.widgets: end = self.widgets.index(orig)
        ws = self.widgets[:end]
        ws.reverse()
        for w in ws:
            if not w.disabled and w.focusable:
                if isinstance(w,Container):
                    if w._previous():
                        return True
                else:
                    self.focus(w)
                    return True
        return False
                
    def next(self,w=None):
        if w != None and w not in self.widgets: return #HACK: maybe.  this happens in windows for some reason...
        
        if self._next(w): return True
        if self.container: return self.container.next(self)
    
    
    def previous(self,w=None):
        if w != None and w not in self.widgets: return #HACK: maybe.  this happens in windows for some reason...
        
        if self._previous(w): return True
        if self.container: return self.container.previous(self)
    
    def resize(self,width=None,height=None):
        #r = self.rect
        #r.w,r.h = 0,0
        ww,hh = 0,0
        if self.style.width: ww = self.style.width
        if self.style.height: hh = self.style.height
        
        for w in self.widgets:
            #w.rect.w,w.rect.h = 0,0
            w.rect.x,w.rect.y = w.style.x,w.style.y
            w.rect.w, w.rect.h = w.resize()
            #w._resize()
            
            ww = max(ww,w.rect.right)
            hh = max(hh,w.rect.bottom)
        return ww,hh

    # Returns the widget with the given name
    def find(self, name):
        for w in self.widgets:
            if (w.name == name):
                return w
            elif (isinstance(w, Container)):
                tmp = w.find(name)
                if (tmp): return tmp
        return None




"""
"""
from .const import *
from . import widget

class Group(widget.Widget):
    """An object for grouping together Form elements.
    
    When the value changes, an gui.CHANGE event is sent. Although note, 
    that when the value is a list, it may have to be sent by hand via 
    g.send(gui.CHANGE).

    """

    _value = None
    widgets = None
    
    def __init__(self,name=None,value=None):
        """Create Group instance.

        Arguments:
        name -- name as used in the Form
        value -- values that are currently selected in the group
    
        """
        widget.Widget.__init__(self,name=name,value=value)
        self.widgets = []
    
    def add(self,w):
        """Add a widget to this group."""
        self.widgets.append(w)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, val):
        oldval = self._value
        self._value = val
        if (oldval != val):
            self._change()
    
    def _change(self):
        self.send(CHANGE)
        if (self.widgets):
            for w in self.widgets:
                w.repaint()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# Implemented using the pybox2d SWIG interface for Box2D (pybox2d.googlecode.com)
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

# Ported from the Cloth test by Paril, originally for Box2CS:
#   http://www.box2d.org/forum/viewtopic.php?f=6&t=6124
#
from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2CircleShape, b2FixtureDef, b2Random, b2Vec2)


def create_cloth(world, segment_count, body_size, position=(0, 30),
                 group_index=-1, bar_height=0.5, base_hz=15,
                 base_damping=0.11, density=0.2):
    segment_w, segment_h = segment_count
    body_spacing_w = body_size * 2
    total_w = body_spacing_w * segment_w
    position = b2Vec2(*position)

    # The static bar at the top which holds the cloth
    bar = world.CreateStaticBody(position=position)
    bar.CreatePolygonFixture(box=(total_w / 2.0 + body_spacing_w,
                                  bar_height / 2.0),
                             groupIndex=group_index)

    box_fixture = b2FixtureDef(shape=b2CircleShape(radius=body_size),
                               groupIndex=group_index, density=density)

    weld_joints = []
    distance_joints = []
    cloth = [[None] * segment_h for x in range(segment_w)]
    for y in range(segment_h):
        pos = position - (total_w / 2.0, y * body_spacing_w)
        for x in range(segment_w):
            pos += (body_spacing_w, 0.0)
            body = world.CreateDynamicBody(position=pos, fixtures=box_fixture)
            cloth[x][y] = body

            if y == 0:
                joint = world.CreateWeldJoint(bodyA=body, bodyB=bar,
                                              anchor=body.position)
                weld_joints.append(joint)

    connect_bodies = []
    for y in range(segment_h):
        for x in range(segment_w):
            if x <= segment_w - 2:
                left_body = cloth[x][y]
                right_body = cloth[x + 1][y]
                connect_bodies.append((left_body, right_body))
            if y > 0:
                left_body = cloth[x][y]
                right_body = cloth[x][y - 1]
                connect_bodies.append((left_body, right_body))

    for bodyA, bodyB in connect_bodies:
        joint = world.CreateDistanceJoint(
            bodyA=bodyA,
            bodyB=bodyB,
            anchorA=bodyA.position,
            anchorB=bodyB.position,
            frequencyHz=base_hz + b2Random(0, base_hz / 2.0),
            dampingRatio=base_damping + b2Random(0.01, base_damping),
        )
        distance_joints.append(joint)

    return cloth, weld_joints, distance_joints


def step_cloth(world, cloth, wind, body_size, segment_count, distance_joints,
               wind_dir=(1, 1), wind_rand=0.0, distance_factor=1.45):
    segment_w, segment_h = segment_count
    body_spacing_w = body_size * 2
    if wind:
        for x in range(segment_w):
            w = (b2Random(wind_dir[0] - wind_rand / 2.0,
                          wind_dir[0] + wind_rand / 2.0),
                 b2Random(wind_dir[1] - wind_rand / 2.0,
                          wind_dir[1] + wind_rand / 2.0))
            cloth[x][-1].linearVelocity += w

    # If any two points are too far from one another, find the joint connecting
    # them and destroy it.
    check_segments = []
    for y in range(segment_h):
        for x in range(segment_w):
            if y > 0:
                check_segments.append((cloth[x][y], cloth[x][y - 1]))
            if x <= segment_w - 2:
                check_segments.append((cloth[x][y], cloth[x + 1][y]))

    thresh = body_spacing_w * distance_factor
    for c1, c2 in check_segments:
        if (c1.worldCenter - c2.worldCenter).length <= thresh:
            continue

        for joint in distance_joints:
            if ((joint.bodyA == c1 and joint.bodyB == c2) or
                    (joint.bodyA == c2 and joint.bodyB == c1)):
                world.DestroyJoint(joint)
                distance_joints.remove(joint)
                break


class Cloth(Framework):
    name = "Cloth"
    description = "(w) Toggle wind"

    def __init__(self):
        super(Cloth, self).__init__()
        self.wind = False
        self.segment_count = (18, 25)
        self.body_size = 0.22

        cloth_info = create_cloth(
            self.world, self.segment_count, self.body_size)
        self.cloth, self.weld_joints, self.dist_joints = cloth_info

    def Keyboard(self, key):
        if key == Keys.K_w:
            self.wind = not self.wind

    def Step(self, settings):
        super(Cloth, self).Step(settings)
        if self.wind:
            self.Print('Wind enabled')
        step_cloth(self.world, self.cloth, self.wind, self.body_size,
                   self.segment_count, self.dist_joints)

if __name__ == "__main__":
    main(Cloth)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from math import ceil, log

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2FixtureDef, b2PolygonShape, b2Vec2)


class Tiles (Framework):
    name = "Tiles"
    description = ('This stress tests the dynamic tree broad-phase. This also'
                   'shows that tile based collision\nis _not_ smooth due to '
                   'Box2D not knowing about adjacency.')

    def __init__(self):
        super(Tiles, self).__init__()

        a = 0.5

        def ground_positions():
            N = 200
            M = 10
            position = b2Vec2(0, 0)
            for i in range(M):
                position.x = -N * a
                for j in range(N):
                    yield position
                    position.x += 2.0 * a
                position.y -= 2.0 * a

        ground = self.world.CreateStaticBody(
            position=(0, -a),
            shapes=[b2PolygonShape(box=(a, a, position, 0))
                    for position in ground_positions()]
        )

        count = 20

        def dynamic_positions():
            x = b2Vec2(-7.0, 0.75)
            deltaX = (0.5625, 1.25)
            deltaY = (1.125, 0.0)
            for i in range(count):
                y = x.copy()
                for j in range(i, count):
                    yield y
                    y += deltaY
                x += deltaX

        for pos in dynamic_positions():
            self.world.CreateDynamicBody(
                position=pos,
                fixtures=b2FixtureDef(
                    shape=b2PolygonShape(box=(a, a)), density=5)
            )

    def Step(self, settings):
        super(Tiles, self).Step(settings)
        cm = self.world.contactManager
        height = cm.broadPhase.treeHeight
        leafCount = cm.broadPhase.proxyCount
        minNodeCount = 2 * leafCount - 1
        minHeight = ceil(log(float(minNodeCount)) / log(2))
        self.Print('Dynamic tree height=%d, min=%d' % (height, minHeight))

if __name__ == "__main__":
    main(Tiles)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2CircleShape, b2EdgeShape, b2FixtureDef, b2PolygonShape,
                   b2Random, b2Vec2, b2_dynamicBody)


class CollisionProcessing (Framework):
    name = "CollisionProcessing"

    def __init__(self):
        super(CollisionProcessing, self).__init__()

        # Tell the framework we're going to use contacts, so keep track of them
        # every Step.
        self.using_contacts = True

        # Ground body
        world = self.world
        ground = world.CreateBody(
            shapes=b2EdgeShape(vertices=[(-50, 0), (50, 0)],)
        )

        xlow, xhi = -5, 5
        ylow, yhi = 2, 35
        random_vector = lambda: b2Vec2(
            b2Random(xlow, xhi), b2Random(ylow, yhi))

        # Small triangle
        triangle = b2FixtureDef(
            shape=b2PolygonShape(vertices=[(-1, 0), (1, 0), (0, 2)]),
            density=1,
        )

        world.CreateBody(
            type=b2_dynamicBody,
            position=random_vector(),
            fixtures=triangle,
        )

        # Large triangle (recycle definitions)
        triangle.shape.vertices = [
            2.0 * b2Vec2(v) for v in triangle.shape.vertices]

        tri_body = world.CreateBody(type=b2_dynamicBody,
                                    position=random_vector(),
                                    fixtures=triangle,
                                    fixedRotation=True,  # <--
                                    )
        # note that the large triangle will not rotate

        # Small box
        box = b2FixtureDef(
            shape=b2PolygonShape(box=(1, 0.5)),
            density=1,
            restitution=0.1,
        )

        world.CreateBody(
            type=b2_dynamicBody,
            position=random_vector(),
            fixtures=box,
        )

        # Large box
        box.shape.box = (2, 1)
        world.CreateBody(
            type=b2_dynamicBody,
            position=random_vector(),
            fixtures=box,
        )

        # Small circle
        circle = b2FixtureDef(
            shape=b2CircleShape(radius=1),
            density=1,
        )

        world.CreateBody(
            type=b2_dynamicBody,
            position=random_vector(),
            fixtures=circle,
        )

        # Large circle
        circle.shape.radius *= 2
        world.CreateBody(
            type=b2_dynamicBody,
            position=random_vector(),
            fixtures=circle,
        )

    def Step(self, settings):
        # We are going to destroy some bodies according to contact
        # points. We must buffer the bodies that should be destroyed
        # because they may belong to multiple contact points.
        nuke = []

        # Traverse the contact results. Destroy bodies that
        # are touching heavier bodies.
        body_pairs = [(p['fixtureA'].body, p['fixtureB'].body)
                      for p in self.points]

        for body1, body2 in body_pairs:
            mass1, mass2 = body1.mass, body2.mass

            if mass1 > 0.0 and mass2 > 0.0:
                if mass2 > mass1:
                    nuke_body = body1
                else:
                    nuke_body = body2

                if nuke_body not in nuke:
                    nuke.append(nuke_body)

        # Destroy the bodies, skipping duplicates.
        for b in nuke:
            print("Nuking:", b)
            self.world.DestroyBody(b)

        nuke = None

        super(CollisionProcessing, self).Step(settings)

if __name__ == "__main__":
    main(CollisionProcessing)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2EdgeShape, b2FixtureDef, b2PolygonShape, b2_dynamicBody,
                   b2_kinematicBody, b2_staticBody)


class BodyTypes(Framework):
    name = "Body Types"
    description = "Change body type keys: (d) dynamic, (s) static, (k) kinematic"
    speed = 3  # platform speed

    def __init__(self):
        super(BodyTypes, self).__init__()

        # The ground
        ground = self.world.CreateBody(
            shapes=b2EdgeShape(vertices=[(-20, 0), (20, 0)])
        )

        # The attachment
        self.attachment = self.world.CreateDynamicBody(
            position=(0, 3),
            fixtures=b2FixtureDef(
                shape=b2PolygonShape(box=(0.5, 2)), density=2.0),
        )

        # The platform
        fixture = b2FixtureDef(
            shape=b2PolygonShape(box=(4, 0.5)),
            density=2,
            friction=0.6,
        )

        self.platform = self.world.CreateDynamicBody(
            position=(0, 5),
            fixtures=fixture,
        )

        # The joints joining the attachment/platform and ground/platform
        self.world.CreateRevoluteJoint(
            bodyA=self.attachment,
            bodyB=self.platform,
            anchor=(0, 5),
            maxMotorTorque=50,
            enableMotor=True
        )

        self.world.CreatePrismaticJoint(
            bodyA=ground,
            bodyB=self.platform,
            anchor=(0, 5),
            axis=(1, 0),
            maxMotorForce=1000,
            enableMotor=True,
            lowerTranslation=-10,
            upperTranslation=10,
            enableLimit=True
        )

        # And the payload that initially sits upon the platform
        # Reusing the fixture we previously defined above
        fixture.shape.box = (0.75, 0.75)
        self.payload = self.world.CreateDynamicBody(
            position=(0, 8),
            fixtures=fixture,
        )

    def Keyboard(self, key):
        if key == Keys.K_d:
            self.platform.type = b2_dynamicBody
        elif key == Keys.K_s:
            self.platform.type = b2_staticBody
        elif key == Keys.K_k:
            self.platform.type = b2_kinematicBody
            self.platform.linearVelocity = (-self.speed, 0)
            self.platform.angularVelocity = 0

    def Step(self, settings):
        super(BodyTypes, self).Step(settings)

        if self.platform.type == b2_kinematicBody:
            p = self.platform.transform.position
            v = self.platform.linearVelocity
            if ((p.x < -10 and v.x < 0) or (p.x > 10 and v.x > 0)):
                v.x = -v.x
                self.platform.linearVelocity = v

if __name__ == "__main__":
    main(BodyTypes)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

# Original C++ version by Daid
#  http://www.box2d.org/forum/viewtopic.php?f=3&t=1473
# - Written for pybox2d 2.1 by Ken
import sys

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2AssertException, b2Color, b2EdgeShape, b2FixtureDef,
                   b2PolygonShape, b2RayCastCallback, b2Vec2)


LASER_HALF_WIDTH = 2
LASER_SPLIT_SIZE = 0.1
LASER_SPLIT_TAG = 'can_cut'


def _polygon_split(fixture, p1, p2, split_size):
    polygon = fixture.shape
    body = fixture.body
    # transform = body.transform

    local_entry = body.GetLocalPoint(p1)
    local_exit = body.GetLocalPoint(p2)
    entry_vector = local_exit - local_entry
    entry_normal = entry_vector.cross(1.0)
    last_verts = None
    new_vertices = [[], []]
    cut_added = [-1, -1]
    for vertex in polygon.vertices:
        # Find out if this vertex is on the new or old shape
        if entry_normal.dot(b2Vec2(vertex) - local_entry) > 0.0:
            verts = new_vertices[0]
        else:
            verts = new_vertices[1]

        if last_verts != verts:
            # if we switch from one shape to the other, add the cut vertices
            if last_verts == new_vertices[0]:
                if cut_added[0] != -1:
                    return []
                cut_added[0] = len(last_verts)
                last_verts.append(b2Vec2(local_exit))
                last_verts.append(b2Vec2(local_entry))
            elif last_verts == new_vertices[1]:
                if cut_added[1] != -1:
                    return []
                cut_added[1] = len(last_verts)
                last_verts.append(b2Vec2(local_entry))
                last_verts.append(b2Vec2(local_exit))

        verts.append(b2Vec2(vertex))
        last_verts = verts

    # Add the cut if not added yet
    if cut_added[0] < 0:
        cut_added[0] = len(new_vertices[0])
        new_vertices[0].append(b2Vec2(local_exit))
        new_vertices[0].append(b2Vec2(local_entry))
    if cut_added[1] < 0:
        cut_added[1] = len(new_vertices[1])
        new_vertices[1].append(b2Vec2(local_entry))
        new_vertices[1].append(b2Vec2(local_exit))

    # Cut based on the split size
    for added, verts in zip(cut_added, new_vertices):
        if added > 0:
            offset = verts[added - 1] - verts[added]
        else:
            offset = verts[-1] - verts[0]
        offset.Normalize()
        verts[added] += split_size * offset

        if added < len(verts) - 2:
            offset = verts[added + 2] - verts[added + 1]
        else:
            offset = verts[0] - verts[len(verts) - 1]
        offset.Normalize()
        verts[added + 1] += split_size * offset

    # Ensure the new shapes aren't too small
    for verts in new_vertices:
        for i, v1 in enumerate(verts):
            for j, v2 in enumerate(verts):
                if i != j and (v1 - v2).length < 0.1:
                    # print('Failed to split: too small')
                    return []

    try:
        return [b2PolygonShape(vertices=verts) for verts in new_vertices]
    except b2AssertException:
        return []
    except ValueError:
        return []


def _laser_cut(world, laser_body, length=30.0, laser_half_width=2, **kwargs):
    p1, p2 = get_laser_line(laser_body, length, laser_half_width)

    callback = laser_callback()
    world.RayCast(callback, p1, p2)
    if not callback.hit:
        return []
    hits_forward = callback.hits

    callback = laser_callback()
    world.RayCast(callback, p2, p1)
    if not callback.hit:
        return []

    hits_reverse = callback.hits

    if len(hits_forward) != len(hits_reverse):
        return []

    ret = []
    for (fixture1, point1), (fixture2, point2) in zip(hits_forward, hits_reverse):
        # renderer.DrawPoint(renderer.to_screen(point1), 2, b2Color(1,0,0))
        # renderer.DrawPoint(renderer.to_screen(point2), 2, b2Color(0,1,0))
        # renderer.DrawSegment(renderer.to_screen(point1), renderer.to_screen(point2), b2Color(0,1,1))
        if fixture1 != fixture2:
            continue

        new_polygons = _polygon_split(
            fixture1, point1, point2, LASER_SPLIT_SIZE)
        if new_polygons:
            ret.append((fixture1, new_polygons))

    return ret


def laser_cut(world, laser_body, length=30.0, laser_half_width=2, **kwargs):
    cut_fixtures = _laser_cut(
        world, laser_body, laser_half_width=LASER_HALF_WIDTH)
    remove_bodies = []
    for fixture, new_shapes in cut_fixtures:
        body = fixture.body
        if body in remove_bodies:
            continue

        new_body = world.CreateDynamicBody(
            userData=LASER_SPLIT_TAG,
            position=body.position,
            angle=body.angle,
            linearVelocity=body.linearVelocity,
            angularVelocity=body.angularVelocity,
        )

        try:
            new_body.CreateFixture(
                friction=fixture.friction,
                restitution=fixture.restitution,
                density=fixture.density,
                shape=new_shapes[1],
            )
        except AssertionError:
            print('New body fixture failed: %s' % sys.exc_info()[1])
            remove_bodies.append(new_body)

        try:
            body.CreateFixture(
                friction=fixture.friction,
                restitution=fixture.restitution,
                density=fixture.density,
                shape=new_shapes[0],
            )

            body.DestroyFixture(fixture)
        except AssertionError:
            print('New fixture/destroy failed: %s' % sys.exc_info()[1])
            remove_bodies.append(body)

    for body in remove_bodies:
        world.DestroyBody(body)


def get_laser_line(laser_body, length, laser_half_width):
    laser_start = (laser_half_width - 0.1, 0.0)
    laser_dir = (length, 0.0)
    p1 = laser_body.GetWorldPoint(laser_start)
    p2 = p1 + laser_body.GetWorldVector(laser_dir)
    return (p1, p2)


def laser_display(renderer, laser_body, length=30.0, laser_color=(1, 0, 0), laser_half_width=2, **kwargs):
    if not renderer:
        return

    p1, p2 = get_laser_line(laser_body, length, laser_half_width)
    renderer.DrawSegment(renderer.to_screen(
        p1), renderer.to_screen(p2), b2Color(*laser_color))


class laser_callback(b2RayCastCallback):
    """This raycast collects multiple hits."""

    def __init__(self, **kwargs):
        b2RayCastCallback.__init__(self, **kwargs)
        self.hit = False
        self.hits = []

    def ReportFixture(self, fixture, point, normal, fraction):
        self.hit = True

        if fixture.body.userData == LASER_SPLIT_TAG:
            self.hits.append((fixture, point))

        self.last_fixture = fixture
        self.last_point = point
        return 1.0


class BoxCutter(Framework):
    name = "Box Cutter"
    description = 'Press (c) to cut'
    move = 0
    jump = 100

    def __init__(self):
        super(BoxCutter, self).__init__()
        # The ground
        self.ground = self.world.CreateStaticBody(
            userData='ground',
            shapes=[
                b2EdgeShape(vertices=[(-50, 0), (50, 0)]),
                b2EdgeShape(vertices=[(-50, 0), (-50, 10)]),
                b2EdgeShape(vertices=[(50, 0), (50, 10)]),
            ]
        )

        self.laser_body = self.world.CreateDynamicBody(
            userData='laser',
            position=(0, 2),
            fixtures=b2FixtureDef(
                density=4.0,
                shape=b2PolygonShape(box=(LASER_HALF_WIDTH, 1))
            )
        )

        for i in range(2):
            self.world.CreateDynamicBody(
                userData=LASER_SPLIT_TAG,
                position=(3.0 + i * 6, 8),
                fixtures=b2FixtureDef(
                    density=5.0,
                    shape=b2PolygonShape(box=(3, 3))
                )
            )

    def Keyboard(self, key):
        if key == Keys.K_c:
            laser_cut(self.world, self.laser_body,
                      laser_half_width=LASER_HALF_WIDTH)

    def Step(self, settings):
        Framework.Step(self, settings)

        laser_display(self.renderer, self.laser_body,
                      laser_half_width=LASER_HALF_WIDTH)

if __name__ == "__main__":
    main(BoxCutter)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2DistanceJointDef, b2EdgeShape, b2FixtureDef,
                   b2PolygonShape)


class Web(Framework):
    name = "Web"
    description = "This demonstrates a soft distance joint. Press: (b) to delete a body, (j) to delete a joint"
    bodies = []
    joints = []

    def __init__(self):
        super(Web, self).__init__()

        # The ground
        ground = self.world.CreateBody(
            shapes=b2EdgeShape(vertices=[(-40, 0), (40, 0)])
        )

        fixture = b2FixtureDef(shape=b2PolygonShape(box=(0.5, 0.5)),
                               density=5, friction=0.2)

        self.bodies = [self.world.CreateDynamicBody(
            position=pos,
            fixtures=fixture
        ) for pos in ((-5, 5), (5, 5), (5, 15), (-5, 15))]

        bodies = self.bodies

        # Create the joints between each of the bodies and also the ground
        #         bodyA      bodyB   localAnchorA localAnchorB
        sets = [(ground,    bodies[0], (-10, 0), (-0.5, -0.5)),
                (ground,    bodies[1], (10, 0),  (0.5, -0.5)),
                (ground,    bodies[2], (10, 20), (0.5, 0.5)),
                (ground,    bodies[3], (-10, 20), (-0.5, 0.5)),
                (bodies[0], bodies[1], (0.5, 0), (-0.5, 0)),
                (bodies[1], bodies[2], (0, 0.5), (0, -0.5)),
                (bodies[2], bodies[3], (-0.5, 0), (0.5, 0)),
                (bodies[3], bodies[0], (0, -0.5), (0, 0.5)),
                ]

        # We will define the positions in the local body coordinates, the length
        # will automatically be set by the __init__ of the b2DistanceJointDef
        self.joints = []
        for bodyA, bodyB, localAnchorA, localAnchorB in sets:
            dfn = b2DistanceJointDef(
                frequencyHz=4.0,
                dampingRatio=0.5,
                bodyA=bodyA,
                bodyB=bodyB,
                localAnchorA=localAnchorA,
                localAnchorB=localAnchorB,
            )
            self.joints.append(self.world.CreateJoint(dfn))

    def Keyboard(self, key):
        if key == Keys.K_b:
            for body in self.bodies:
                # Gets both FixtureDestroyed and JointDestroyed callbacks.
                self.world.DestroyBody(body)
                break

        elif key == Keys.K_j:
            for joint in self.joints:
                # Does not get a JointDestroyed callback!
                self.world.DestroyJoint(joint)
                self.joints.remove(joint)
                break

    def FixtureDestroyed(self, fixture):
        super(Web, self).FixtureDestroyed(fixture)
        body = fixture.body
        if body in self.bodies:
            print(body)
            self.bodies.remove(body)
            print("Fixture destroyed, removing its body from the list. Bodies left: %d"
                  % len(self.bodies))

    def JointDestroyed(self, joint):
        if joint in self.joints:
            self.joints.remove(joint)
            print("Joint destroyed and removed from the list. Joints left: %d"
                  % len(self.joints))

if __name__ == "__main__":
    main(Web)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2CircleShape, b2EdgeShape, b2FixtureDef)


class Restitution (Framework):
    name = "Restitution example"
    description = "Note the difference in bounce height of the circles"

    def __init__(self):
        super(Restitution, self).__init__()

        # The ground
        ground = self.world.CreateStaticBody(
            shapes=b2EdgeShape(vertices=[(-20, 0), (20, 0)])
        )

        radius = 1.0
        density = 1.0
        # The bodies
        for i, restitution in enumerate([0.0, 0.1, 0.3, 0.5, 0.75, 0.9, 1.0]):
            self.world.CreateDynamicBody(
                position=(-10 + 3.0 * i, 20),
                fixtures=b2FixtureDef(
                    shape=b2CircleShape(radius=radius),
                    density=density, restitution=restitution)
            )

if __name__ == "__main__":
    main(Restitution)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2CircleShape, b2EdgeShape, b2FixtureDef, b2PolygonShape,
                   b2_linearSlop)


class OneSidedPlatform (Framework):
    name = "One-sided Platform"
    description = ("Try to move the shape with the mouse through the platform "
                   "from all directions")

    def __init__(self):
        super(OneSidedPlatform, self).__init__()

        # The ground
        ground = self.world.CreateBody(
            shapes=b2EdgeShape(vertices=[(-20, 0), (20, 0)])
        )

        # The platform
        half_height = 0.5
        ypos = 10
        body = self.world.CreateBody(
            position=(0, ypos),
            shapes=b2PolygonShape(box=(3, half_height))
        )
        self.platform = body.fixtures[0]

        # The circular character
        self.character_radius = 0.5
        body = self.world.CreateDynamicBody(
            position=(0, 12),
            fixtures=b2FixtureDef(shape=b2CircleShape(
                radius=self.character_radius), density=1.0),
        )

        self.character = body.fixtures[0]
        body.linearVelocity = (0, -50)

        self.bottom = ypos - half_height  # The bottom of the platform
        self.top = ypos + half_height    # The top of the platform
        self.state = 'unknown'

    def PreSolve(self, contact, oldManifold):
        super(OneSidedPlatform, self).PreSolve(contact, oldManifold)

        #  Make sure we're dealing with the platform and the character
        if (contact.fixtureA != self.platform and
                contact.fixtureA != self.character):
            return
        if (contact.fixtureB != self.platform and
                contact.fixtureB != self.character):
            return

        # If below the top of the platform, disable the collision response
        pos = (self.character.body.position.y - self.character_radius +
               3.0 * b2_linearSlop)
        if pos < self.top:
            contact.enabled = False

if __name__ == "__main__":
    main(OneSidedPlatform)



from Box2D import *

class Segfaulter:
    def __init__(self):
        self.world = b2World(contactListener=b2ContactListener())
        body1 = self.world.CreateDynamicBody(position=(0, 0))
        body1.CreateCircleFixture()
        body2 = self.world.CreateDynamicBody(position=(0, 0))
        body2.CreateCircleFixture()

    def step(self):
        self.world.Step(1 / 30, velocityIterations=1, positionIterations=1)


def test_issue_110():
    segfaulter1 = Segfaulter()
    segfaulter2 = Segfaulter()
    for i in range(10):
        segfaulter1.step()
        segfaulter2.step()



"""Constants.

From pygame:
    QUIT
    MOUSEBUTTONDOWN
    MOUSEBUTTONUP
    MOUSEMOTION
    KEYDOWN

PGU specific:
    ENTER
    EXIT
    BLUR
    FOCUS
    CLICK
    CHANGE
    OPEN
    CLOSE
    INIT

Other:
    NOATTR

"""
import pygame

from pygame.locals import QUIT, MOUSEBUTTONDOWN, MOUSEBUTTONUP, MOUSEMOTION, KEYDOWN, USEREVENT
ENTER = pygame.locals.USEREVENT + 0
EXIT = pygame.locals.USEREVENT + 1
BLUR = pygame.locals.USEREVENT + 2
FOCUS = pygame.locals.USEREVENT + 3
CLICK = pygame.locals.USEREVENT + 4
CHANGE = pygame.locals.USEREVENT + 5
OPEN = pygame.locals.USEREVENT + 6
CLOSE = pygame.locals.USEREVENT + 7
INIT = 'init'

class NOATTR: 
    pass


